/*******************************************************************
 *
 *    DESCRIPTION: Init and language support 
 *				   Part of TeX support for CodeWRight (TEX.DLL).
 *
 *    AUTHOR: 	Gilbert Ritschard, Department of Econometrics
 *				University of Geneva, CH-1211 Geneva 4
 *				e-mail: ritschar@uni2a.unige.ch
 *
 *    HISTORY:  1.0 	6 sept., 1994, created
 *				1.0a	12 oct., 1994, modified (math coloring on at startup)
 *				2.0		27 Dec., 1997, Win32 version for CW32 v.5.0
 *
 *		Part of the code is taken from "c.c" (C)CodeWRight
 *
 *******************************************************************/


#include <windows.h>
#include <string.h>
#include <ctype.h>
#include "exports.h"
//#define __CWSTART_STRID__
#include "cwstart.h"
#include "tex.h"

#define CW50	1


LIBMAIN    /* defines the LibMain() function  */

int  test = 1;

 // #define end_of_line(ch)\
 // (( ((ch) == EOL1_CHAR)||( (ch) == EOL2_CHAR) || ( (ch) == EOF_CHAR ) )\
 // ? (TRUE) : (FALSE))


#define MAX_BUF_LEN    100
//static char buf[MAX_BUF_LEN + 1];

static LPVOID _tex_hash_names        = NIL; /* pointer to the hash table containing */
                                            /* all 'TEX' keyword names. */

static LPVOID _tex_hash_names_1      = NIL; /* pointer to optional hash table containing */
static int _tex_hash_size_1          = 47;  /* user defined keyword names and size of
                                            /* table. When created these key
                                            /* are set to Color Alternate5 */           
                        

static LPVOID _tex_hash_names_2      = NIL; /* pointer to optional hash table containing */
static int _tex_hash_size_2          = 47;  /* user defined keyword names and size of
                                            /* table. When created these key
                                            /* are set to Color Alternate6 */              

static LPVOID _tex_hash_templates    = NIL; /* pointer to the hash table containing */
                                            /* an abbreviated name and its template */

/* Define a structure to hold an abbreviation and the template to be
 * expanded when the abbreviation is entered into the buffer.
 * This structure will be the element in the hash table _tex_hash_templates.
 */
typedef struct _LanguageTemplate {         
	LPSTR abbrev;
	LPSTR template;
} LanguageTemplate;


/* This string contains all keyword names which will be indented an extra level.
 */
LPSTR TEX_INDENT_STR = "\377\\begin{enumerate}\377\\begin{itemize}\377\\begin{list}\377";

/*
 * _tex_templates[] contains the keyword abbreviation and the template for the
 * abbreviation.  To add a new name, insert it before the NIL entry or
 * call _tex_assign_template().
 *
 * The special characters within the template are:
 *
 *     \n - new line
 *     &  - cursor position after template insertion
 *     @  - backspace
 *     \c - insert 'c' literally
 */
LanguageTemplate _tex_templates[] = {
//    Abbreviation                       Template
	{ "{",	"{&}" },
	{ "\\begin|\\begin{",	"\\\\begin{&}\n\n\\\\end{}" },
	{ "\\begin{array|\\array",		"\\\\begin{array}{&}\n\n\\\\end{array}" },
	{ "\\begin{center|\\cent|\\center",		"\\\\begin{center}\n&\n\\\\end{center}" },
	{ "\\begin{doc|\\begin{document",		"\\\\begin{document}\n&\n\\\\end{document}" },
	{ "\\begin{description|\\descr|\\description",	"\\\\begin{description}\n\t\\\\item&\n@ \\\\end{description}" },
	{ "\\begin{enumerate|\\enum|\\enumerate",	"\\\\begin{enumerate}\n\t\\\\item&\n@ \\\\end{enumerate}" },
	{ "\\begin{equation|\\equ",	"\\\\begin{equation}\n&\n\\\\end{equation}" },
	{ "\\begin{eqnarray|\\eqna",	"\\\\begin{eqnarray}\n&\n\\\\end{eqnarray}"},
	{ "\\begin{eqnarray*|\\eqna*",	"\\\\begin{eqnarray*}\n&\n\\\\end{eqnarray*}" },
	{ "\\begin{figure",		"\\\\begin{figure}[htb]\n&\n\\\\end{figure}" },
	{ "\\begin{itemize|\\itemize",	"\\\\begin{itemize}\n\t\\\\item&\n@ \\\\end{itemize}" },
	{ "\\begin{quote",		"\\\\begin{quote}\n&\n\\\\end{quote}" },
	{ "\\begin{letter",		"\\\\begin{letter}\n\t{&\n}\n\n@ \\\\opening{}\n\n\\\\closing{\\\\ }\n\\\\end{letter}"},					
	{ "\\begin{minipage",	"\\\\begin{minipage}[t]{&}\n\n\\\\end{minipage}" },
	{ "\\begin{table",		"\\\\begin{table}[htb]\n&\n\\\\end{table}" },
	{ "\\begin{tabular|\\tabu|\\tabular",	"\\\\begin{tabular}{&}\n\n\\\\end{tabular}" },
	{ "\\begin{thebib|\\begin{thebibliography",	"\\\\begin{thebibliography}{MM}\n\\\\bibitem&\n\\\\end{thebibliography}" },
	{ "\\begin{slide|\\sli|\\slide", "@ \\%==============================\n\\\\begin{slide}\n&\n\\\\end{slide}" },
	{ "\\[",  				"@ \\\\[\n\t&\n@ \\\\]"},
	{ "\\documentc|\\documentclass|\\latex",	"\\\\documentclass[12pt]{article}\n\\\\usepackage[&]{}\n\n\\\\begin{document}\n\n\\\\end{document}" },
	{ "\\documents|\\documentstyle|\\209",	"\\\\documentstyle[12pt]{article}\n&\n\\\\begin{document}\n\n\\\\end{document}" },
	{ "\\graphics|\\includegraphics|\\psfig","\\\\includegraphics[width=\\\\linewidth]{&}"},
	{ "\\begin{sideways\\sideways|\\sidew", "\\\\begin{sideways}\n\\\\parbox[t]{.94\\\\textheight}{\n&\n}\n\\\\end{sideways}"},
	{ "\\puttext","\\\\put(0,-1){\\\\parbox[t]{0em}{\\\\makebox[0pt][c]{&}}}"},
	{ NIL,       NIL },
};
	/*{ "\\psfig",			"\\\\psfig{figure=&.eps,width=\\\\linewidth,height=}"},
	 */

/* Define the list of TeX keyword names to appear in
 * the keyword color if language coloring is enabled.
 */
LPSTR _tex_names[] = {
    // TeX  commands
    // ----------------------
	"\\\\begin",	"\\\\end", "\\\\[A-Za-z]", "ereur",	
	NIL, };


/* private functions */
static long _tex_in_comment( long offset );
//static long _tex_find_comment_start( void );
static int	_tex_find_math_end( int );
static int	_tex_check_math_start( void );
static int	_tex_check_math_end( int );
//static int	_tex_find_math_inline( long );
static long _tex_find_math_start( long startoff );
static long _tex_find_beg_firstline( long firstline );

BOOL tex_is_string_token( long* start, long* end );
BOOL tex_is_number_token( long* start, long* end);
BOOL tex_is_keyword_token( int* table, long* start, long* end );

/* public functions */
void DLL _tex_init( void );
BOOL DLL _tex_expand( void );
long DLL _tex_language_colors_update( long firstline, long lastline );
long DLL _tex_language_colors( long numlines );
long DLL _sty_language_colors_update( long firstline, long lastline );
void DLL _sty_language_colors( long numlines );
void DLL _tex_assign_template( LPSTR abbrev, LPSTR template );
void DLL _tex_add_keyword( LPSTR keyword );
BOOL DLL _tex_indent( void );

WORD  verbch = '+' ;


/*******************************************************************
 * _init
 *
 * PARAMETERS:  none.
 *
 * DESCRIPTION: Initialization function.  Called when dll is loaded.
 *              Calls the intialization function for this module. 
 *              
 * RETURN:      None.
 *******************************************************************/
void DLL 
_init()
{
	_tex_init();
}


/*******************************************************************
 *  _tex_init()
 *
 *  PARAMETERS:  none.
 *
 *  DESCRIPTION: Initializes this module by exporting any required
 *   functions and creating the hash tables needed to support 
 *   TEX language template expansion and TEX keyword coloring.
 *
 *  RETURN:      none.
 * 
 *  NOTE:        It is recommended that the hash table length be a
 *   prime number to maximize utilization of the hash table.
 ******************************************************************/
void DLL
_tex_init( void )
{
	static BOOL firstTime = TRUE;
	int    i;
//	LPSTR hashStr;

	if (firstTime)
	{
		LibExport( "BOOL DlgTexSetup HWND" );
		LibExport( "UINT TeXCommand HWND UINT" );
		LibExport( "UINT ExecTeX1 LPSTR" );
		LibExport( "UINT ExecTeX2 LPSTR" );
		LibExport( "UINT ExecPrint1 LPSTR" );
		LibExport( "UINT ExecPrint2 LPSTR" );
		LibExport( "UINT ExecPrint3 LPSTR" );
		LibExport( "UINT ExecPrint4 LPSTR" );
		LibExport( "UINT ExecSpell1 LPSTR" );
		LibExport( "UINT ExecSpell2 LPSTR" );

		LibExport( "void TeXColorMath int" );
		LibExport( "int _TeXErrorInfo void" );

		LibExport( "void ToggleTexWrapEnable void" );
		LibExport( "void TexWrapParagraph void" );
		LibExport( "void TexWrapAll void" );
		LibExport( "void TextToTeX int" );
		LibExport( "void TeXToText int" );
		LibExport( "long _tex_ww_handler long long" );
		EventRegister( EVENT_CHAR_INSERTED,  EVENT_FIRST, "_tex_ww_handler" );

		LibExport( "long _tex_buf_current_handler long long" );
		EventRegister( EVENT_NEW_EDITFILE,  EVENT_LAST, "_tex_buf_current_handler" );
		EventRegister( EVENT_BUFFER_CURRENT,  EVENT_LAST, "_tex_buf_current_handler" );
		EventRegister( EVENT_FILENAME_CHANGED,  EVENT_LAST, "_tex_buf_current_handler" );

		LibExport( "_tex_buf_uncurrent_handler long long" ); 
		EventRegister( EVENT_BUFFER_UNCURRENT,  EVENT_NORMAL, "_tex_buf_uncurrent_handler" );
		//EventRegister( EVENT_WINDOW_UNCURRENT,  EVENT_NORMAL, "_tex_buf_uncurrent_handler" );

		LibExport( "long _tex_close_handler long long" );
		EventRegister( EVENT_EXIT_SHUTDOWN,  EVENT_NORMAL, "_tex_close_handler" );

		LibExport( "BOOL _tex_expand");
		LibExport( "_tex_assign_template LPSTR LPSTR");
		LibExport( "_tex_add_keyword LPSTR");
		LibExport( "BOOL _tex_indent( void ) ");
		LibExport( "long _tex_language_colors_update long long");
		LibExport( "_tex_language_colors long");
		LibExport( "long _sty_language_colors_update long long");
		LibExport( "_sty_language_colors long");
        LibExport( "_tex_hash_size int int" );
		LibExport( "_tex_routines" );

		_AddErrorInfoName( "_TeXErrorInfo" );

#if CW50
  	    // New for CW 5.0, commented out for now, until 5.0 is released 
		// setup for Outine

		ExtAddOutlineParser("tex", OUTLINE_TYPE_CLASS, "Package", 
		  PARSER_FLAG_ALPHA|PARSER_FLAG_NUMERIC|PARSER_FLAG_UNDERSCORE|PARSER_FLAG_ROUTINE|PARSER_FLAG_CASE_SENSITIVE,
 		  " {}\\*-\\?\\'!\\[\\],",
 		  "(^[ \\t]?\\\\\\cdocument.*})|(^[ \\t]*\\\\\\cusepackage.*})");

		ExtAddOutlineParser("tex", OUTLINE_TYPE_FUNCTION, "Section",
		  PARSER_FLAG_ALPHA|PARSER_FLAG_NUMERIC|PARSER_FLAG_UNDERSCORE|PARSER_FLAG_ROUTINE|PARSER_FLAG_CASE_SENSITIVE,
 		  " {}*-\\'\\?!\\.:,\\[\\]\\t",
 		  "(^[ \\t]*\\\\\\csection.*[}\\n])|(^[ \\t]*\\\\\\cpart.*[}\\n])|(^[ \\t]*\\\\\\cchapter.*[}\\n])");
	   
		ExtAddOutlineParser("tex", OUTLINE_TYPE_FUNCTION, "Subsection",
		  PARSER_FLAG_ALPHA|PARSER_FLAG_NUMERIC|PARSER_FLAG_UNDERSCORE|PARSER_FLAG_ROUTINE|PARSER_FLAG_CASE_SENSITIVE,
 		  " {}*-\\'\\?!\\.:,\\[\\]\\t",
 		  "(^[ \\t]*\\\\\\csubs?u?b?section.*[}\\n])|(^[ \\t]*\\\\\\c.*paragraph.*[}\\n])");
	   
		ExtAddOutlineParser("tex", OUTLINE_TYPE_PROCEDURE, "Begin",
		  PARSER_FLAG_ALPHA|PARSER_FLAG_NUMERIC|PARSER_FLAG_UNDERSCORE|PARSER_FLAG_ROUTINE|PARSER_FLAG_CASE_SENSITIVE,
 	 	  " {}*-\\'\\?!\\.,",
 	 	  "^[ \\t]*\\\\\\cbegin{.*}");

		ExtAddOutlineParser("tex", OUTLINE_TYPE_VARIABLE, "Textbf",
		  PARSER_FLAG_ALPHA|PARSER_FLAG_NUMERIC|PARSER_FLAG_UNDERSCORE|PARSER_FLAG_ROUTINE|PARSER_FLAG_CASE_SENSITIVE,
 		  " (){}*-\\'\\?!\\.:,\\[\\]\\t\\$",
 		  "(^[ \\t]*\\\\textbf{\\c.+[}\\n])");

		ExtAddOutlineParser("tex", OUTLINE_TYPE_MACRO, "Include",
		  PARSER_FLAG_ALPHA|PARSER_FLAG_NUMERIC|PARSER_FLAG_UNDERSCORE|PARSER_FLAG_ROUTINE|PARSER_FLAG_CASE_SENSITIVE,
 		  " %{}\\.:,-\\[\\]\\\\=",
 		  "(^[ \\t]*\\\\\\cinclude.+[}\\n])|(^[ \\t]*\\\\\\cinput .+[ \\n])");
#endif

//gr97        /* Create a hash table to hold all of the 'TEX' keyword names.
//gr97         */
//gr97        _tex_hash_names = HashCreateTable( 701/* a prime # */, sizeof( LPSTR ), HASH_IGN_CASE );
//gr97        i = 0;
//gr97
//gr97        while ( _tex_names[ i ] != NIL )
//gr97        {
//gr97            LPSTR XFAR *lpf = (LPSTR XFAR *)HashGetEntry( _tex_hash_names, 
//gr97                _tex_names[i], strlen( _tex_names[i] ) );
//gr97            if (lpf)
//gr97            {
//gr97                *lpf = StrNew( _tex_names[i] );
//gr97                i++;
//gr97            }
//gr97        }
//gr97		hashStr = HashFindFirst(_tex_hash_names, NIL);
//gr97		while (hashStr) 
//gr97		    hashStr = HashFindNext();


        /* Create a hash table to hold all of the 'TEX' template names.
         */
        _tex_hash_templates = HashCreateTable( 301 /* a prime # */, sizeof( LPSTR ), HASH_IGN_CASE );
        i = 0;

        while ( _tex_templates[ i ].abbrev != NIL )
        {
            _tex_assign_template( _tex_templates[i].abbrev, _tex_templates[i].template );
            i++;
        }
        // Setup the %10 template macro to hold the double backslash characters
        ExtSetTemplateMacro( 10, "\\\\", NIL );

		CompilerNewExt( ".tex" );
		CompilerNewExt( EXT_TEX_CMD );
		CompilerNewExt( EXT_TEX_FLAGS );

		//CompilerAddCmd( "TeX Flags",  "", 1L, 
		//				"", 0L, "", 0L, "", 0L, "", 0L, "", 0L, 
		//				"", "", "", "", ""); 
		CompilerAssign( DESC_TEX_FLAGS, EXT_TEX_FLAGS );

		TexMenuInsert ();

		_currentBuf_wrap = FALSE;
		firstTime = FALSE;

   	}

}

/*
 ** _tex_buf_current_handler
 *
 *  FILENAME: D:\CW32\TEX\TEXMENU.C
 *
 *  PARAMETERS:	 long eventnum, long chptr
 *
 *  DESCRIPTION:  Initiates the flags
 *
 *  RETURNS:
 *
 */

long DLL
_tex_buf_current_handler( long eventnum, long chptr ) 
{
 	DWORD flags = TexQFlags();

//	LPSTR		ext;
//	long		len;
//
//	ext = TransformFilename( BufQFilename(), "%e" );
//
//	len = StrMatch( "tex" ,ext , SEARCH_IGCASE, &len ) ;
//	if ( len )
//	{
//		_tex_language_colors(-1);
//		MsgNotify(StrItoA(len,10));   
//	}

	if ( BufQSysFlags() & BUFFER_WORDWRAP )
	{
		BufSetSysFlags( BufQSysFlags() & ~BUFFER_WORDWRAP );
		_currentBuf_wrap = TRUE;
	}

	CheckMenuItem( MenuQHandle() , IDM_TEX_MAIN, 
				( flags & FLAG_TEX_MAIN ? MF_CHECKED : MF_UNCHECKED));
	CheckMenuItem( MenuQHandle() , IDM_TEX_WORD_WRAP, 
				( flags & FLAG_TEX_WRAP ? MF_CHECKED : MF_UNCHECKED));
	CheckMenuItem( MenuQHandle() , IDM_TEX_MATHCOLOR, 
				( flags & TEX_COLOR_MATH ? MF_UNCHECKED : MF_CHECKED ));

	return 1L;
}

/*
 ** _tex_buff_uncurrent_handler
 *
 *  FILENAME: D:\CW32\TEX\TEX.C
 *
 *  PARAMETERS:	 long eventnum, long chptr
 *
 *  DESCRIPTION:  Saves the flags
 *
 *  RETURNS:
 *
 */

long DLL
_tex_buf_uncurrent_handler( long eventnum, long chptr ) 
{
	if ( _currentBuf_wrap )
		BufSetSysFlags( BufQSysFlags() | BUFFER_WORDWRAP );
		_currentBuf_wrap = FALSE;
	return 0L;
}

/*
 ** _tex_close_handler
 *
 *  FILENAME: D:\CW32\TEX\TEX.C
 *
 *  PARAMETERS:	 long eventnum, long chptr
 *
 *  DESCRIPTION:  Saves the flags
 *
 *  RETURNS:
 *
 */

long DLL
_tex_close_handler( long eventnum, long chptr ) 
{
	ConfigUpdate(  CFG_UPDATE_COMPILER_SETUP );
	return 0L; 
}


/*
 ** TexQFlags
 *
 *  FILENAME: D:\CW32\TEX\TEXMENU.C
 *
 *  PARAMETERS:	 none
 *
 *  DESCRIPTION:
 *
 *  RETURNS:  TeX flags
 *
 */
DWORD TexQFlags( void )
{
	LPErrorInfo lpErrinfo;
	
	//CompilerAssign( DESC_TEX_FLAGS, EXT_TEX_FLAGS );
	lpErrinfo = _ExtDescriptionEntry( DESC_TEX_FLAGS );

	// We use the compileDFlags of ".@@eX" as TeX Flags.
	return (DWORD) lpErrinfo->compileDFlags; 
}


/*
 ** TexSetFlags
 *
 *  FILENAME: D:\CW32\TEX\TEXMENU.C
 *
 *  PARAMETERS:	DWORD flags
 *
 *  DESCRIPTION:  Sets TeX flags to "flags"
 *
 *  RETURNS: flags
 *
 */
DWORD TexSetFlags( DWORD flags )
{
	LPErrorInfo lpErrinfo;
	//LPSTR empty = "";

	//CompilerAssign( DESC_TEX_FLAGS, EXT_TEX_FLAGS );
	lpErrinfo = _ExtDescriptionEntry( DESC_TEX_FLAGS );

	// We use the compileDFlags of "TeX Flags" associated to ".@@eX" as TeX Flags.
	 lpErrinfo->compileDFlags = (WORD) flags;
	 lpErrinfo->cmdCompileDbg = StrNew( "TeXFlags" );

	return (DWORD) lpErrinfo->compileDFlags;
}


/*******************************************************************
 * _tex_hash_size()
 *******************************************************************/
void DLL
_tex_hash_size( int ht, int size )
{
    if (size > 0)
    {
        if (ht == 1)
            _tex_hash_size_1 = size;
        else if (ht == 2)
            _tex_hash_size_2 = size;
    }
}


/*
 ** _tex_expand()
 *
 *  DESCRIPTION:
 *    Attempts to perform TeX language template expansion.  This function
 *    is normally bound to the <SPACE> key.  When invoked, it reads the 
 *    previous word.  If the word is in the list of defined templates,
 *    the template is extracted and inserted.  Some characters in the 
 *    templates have special meanings. They are:
 *
 *       \n - new line
 *       &  - cursor position after template insertion
 *       @  - backspace
 *       \  - take next character literally
 */

BOOL DLL
_tex_expand( void )
{
	LPVOID htEntry;
	LPSTR  _tex_string = NULL;
	LPSTR  str;
	int    i = 0;

    // Save the current buffer position on the stack
    MarkSavePos();

    // Use spaces - disable the use of tab characters
    BufSetTabUsage( FALSE );


	/* Check to see if we are currently positioned inside a TeX comment
	 */
	if (_tex_in_comment( -1L ) != -1)
		goto _tex_expand_incomplete;

    // Allocate a buffer and read entire line of starting cursor position
    // The maximum string length returned by BufReadStr is 32767 chars 
	str = BufReadStr( 65535 );
 //gr 	while (str[i] && !isspace((BYTE)str[i]))
 //gr 		i++;
 //gr 	str[i] = '\0';

    // Strip line to first word 
    _tex_string = StrTrim( str, " \t" );
	StrFree( str );

	if (!_tex_string)
		goto _tex_expand_incomplete;

	/* Search for a keyword match in the hash table.
	 */
	htEntry = HashFindEntry( _tex_hash_templates, _tex_string, strlen( _tex_string ) );

	if (htEntry)
	{
		/* A template exists for the current keyword, delete the keyword
		 * and insert the template.
		 */
        // Remove the last saved position from the stack
		MarkDropPos( 1 );

		str = *(LPSTR*)htEntry;
		BufDelChar( lstrlen( _tex_string ));

		_ext_expand_template( str );

	} 
    else   // no template found for the current word.
		goto _tex_expand_incomplete;

    // delete the buffer 
	StrFree( _tex_string );
	return TRUE;

_tex_expand_incomplete:
	StrFree( _tex_string );
	MarkRestorePos();

	return FALSE;
}

long DLL 
_sty_language_colors_update( long firstline, long lastline )
{
	return _tex_language_colors_update( firstline, lastline );
}


/*
** _tex_language_colors_update()
*
*  This function does an intermediate update of the colors within
*  a range of line numbers (firstline and lastline).  It must also
*  determine if the first and last line specified are within a
*  comment and act appropriately.
*/

long DLL
_tex_language_colors_update( long firstline, long lastline )
{
	long count = 0;
	long startoff;
	long mathstartoff;
	long col;

	if (firstline && lastline)
	{
		//col = BufQCurrentCol() - 1L;
		col = 1L;
		_PosInit( 0 );
		_PosNextLine(lastline);
		lastline = _PosQLine();
		if ( lastline < firstline )
		   firstline = lastline ; //	return = 0L;	
		
		startoff = _tex_find_beg_firstline( firstline ) + col ;
		_PosInit( startoff );

 // 		MsgNotify(StrAppend(
 // 		    StrAppend(StrItoA( col, 10), StrNew(" col ")),		
 // 		    StrAppend(StrItoA( startoff, 10), StrNew(" startoff "))
 // 		    ));		
		
		/* If mathcoloring is on we redefine the firstline and lastline
		 * to fit the math sequence.
		 */
		if ( !(TexQFlags() & TEX_COLOR_MATH ) )
		{
			/* if current line ends in math mode,
			 * we color to the end of the buffer
			 */
			_PosInit( startoff );
			_PosNextLine(0L);
			_PosNextChar();
			if( _PosQColor() == ColorAlternate4(-1)
			  				|| _PosQColor() == ColorAlternate4(-1) + 0xf0 
			  				|| _PosQColor() == ColorAlternate5(-1) 
			  				|| _PosQColor() == ColorAlternate5(-1) + 0xf0) 
				lastline = 0x7FFFFFFFL;

			_PosInit( startoff );
			mathstartoff = _tex_find_math_start( startoff );
			
			/* color from the start of the math sequence
			 */
			_PosInit( mathstartoff );
			firstline = _PosQLine();

			/* Compute the col position of the mathstartoff
			 */
			col = mathstartoff - _tex_find_beg_firstline( firstline ) + 1;
			startoff = mathstartoff;
		}

		/* Reset the attributes for the line range to be
		 * recolored with _tex_language_colors.
		 */
		SysSetSystemFlags( SYSTEM_DELAY_ATTR_CHANGES );
		AttrSetColor( firstline, firstline, col, 0x7FFFFFFFL, 0 );
		if ( lastline > firstline )
			AttrSetColor( firstline + 1, lastline, 1L, 0x7FFFFFFFL, 0 );
		_PosInit( startoff );
		count = _tex_language_colors( lastline - firstline + 1 );

		/* if a math sequence ends after lastline
		 * we recolor until the end of the buffer.
		 */
		if ( count > lastline )
		{
			lastline = 0x7FFFFFFFL;
			AttrSetColor( firstline + 1, lastline, 1L, 0x7FFFFFFFL, 0 );
			_PosInit( startoff );
			_tex_language_colors( lastline - firstline + 1 );
		}

		/* Allow changes to be redraw 
		 */
 		SysClrSystemFlags( SYSTEM_DELAY_ATTR_CHANGES );
	 	//WinRedraw( WinQCurrentWindow() );
 		WinRedrawLines( WinQCurrentWindow(), firstline, lastline );
 //		 	WinRedraw( WinQCurrentWindow() );
	}

	else if ( !(TexQFlags() & TEX_COLOR_MATH ) )
		TeXColorMath(1);
	else
		TeXColorMath(0);

	return count;
}



void DLL 
_sty_language_colors( long numlines )
{
	_tex_language_colors( numlines );
}

/*****************************************************************
 ** _tex_language_colors()
 *
 * PARAMETERS:  long numlines - number of lines to process
 *
 * DESCRIPTION: Sets a block of text according to TEX language
 *   syntax.  If the given number of lines equals (-1), coloring
 *   begins at the top of the buffer, otherwise coloring begins
 *   at the current _Pos position.
 *
 *   Each line of text is processed.  First the line is checked for
 *   a '%'.  If found the end of the line is colored
 *   as a comment and we move on. The line is parsed and the token
 *   are colored according to its token type, string, number
 *   or TEX language keyword (any alpha-string preceeded by a "\" or
 *   any single character preceeded by a "\".)
 *   The two variables start and end are
 *   maintained while each line is parsed.  Start holds the _Pos offset
 *   of the first character of the token.  End maintains the _Pos offset
 *   one past the last character of the token.
 *
 * RETURN:      none.
 * 
 * NOTE:        It is assumed that if numlines != -1, that _PosInit()
 *   has been called and numlines is relative to the current 
 *   position.
 *
 * TODO:        check additional keyword tables.
 ******************************************************************/
long DLL
_tex_language_colors( long numlines )
{
	long  startoff;
	long  startline;
	long  endoff;
	long  endmathoff = 0L;
	long  inc;
	long  endmathline = 0L;
	long  endverbline  ;
	long  endline;
	UINT  ch;
	UINT  lastch;
//	LPSTR buf;
	int   i = 0;
	int   comment_color    = ColorComments( -1 );
	int   keyword_color    = ColorKeywords( -1 );
	long  maxlines         = ExtCommentSearchLimit( -1 );
//	int   quote_color      = ColorAlternate1( -1 );
	int   number_color     = ColorAlternate2( -1 );
	int   brace_color      = ColorAlternate3( -1 );
	int   math_color       = ColorAlternate4( -1 );
	int   verbatim_color   = ColorAlternate5( -1 );
	int   _tex_mathstyle 	= 0;
	//int   start_line_mathstyle 	= 0;
	//BOOL  _tex_firstmath	= TRUE;

									   
//	buf = (LPSTR)MemAlloc( MAX_BUF_LEN );

	/* Start at the top of the buffer or current position 
	 */

	if (numlines == -1L )
	{
		startoff = 0;  
		numlines = 0x7FFFFFFFL;
	}
	else
	{
		//_PosInit(-1);
		startoff  = max(_PosQOffset(), 0);
		if (startoff)
		{
			endoff    = _tex_in_comment( startoff );
			if ( endoff > -1 ) // we are in a comment: start at its begening
			{
				startoff = endoff + 1;
				//numlines = 1;
			}
		}
	}
	_PosInit( startoff );
	startline = _PosQLine();
	endline = startline + numlines - 1;
	ch = _PosCurrentChar();
	//startoff = _PosQOffset();
	//_PosInit( startoff );

	//maxlines = max( numlines, maxlines );

	while ((ch != EOF_CHAR) && (numlines > 0))
	{
		lastch = _PosPrevChar();
		if ( lastch != EOF_CHAR )
			_PosNextChar();

		if (ch == '\n')	// newline
		{
			numlines--;
//			ch = _PosNextChar();
//			continue;
		}


		if ( (!(TexQFlags() & TEX_COLOR_MATH )) && (lastch != '\\') )
		{


/* math mode:
 *  _tex_mathstyle = 	0 : not math mode
 *						1 : $ ... $
 *						2 : $$ ... $$
 *						3 : \[ ... \]
 *						4 : \( ... \)
 *						5 : \begin{equation} ... \end{equation}
 *						6 : \begin{eqnarray} ... \end{eqnarray}
 *						7 : \begin{eqnarray*} ... \end{eqnarray*}
 *
 *						10 : \begin{verbatim} ... \end{verbatim}
 *						10+ch : \verb(ch) ... (ch)
 */

		
			if ( (ch == '$') || (ch == '\\') )
			{
	            startoff = _PosQOffset();
				_PosInit( startoff );
							
				if (startoff > endmathoff)
				{
					_tex_mathstyle = _tex_check_math_start();
					if(_tex_mathstyle) _PosNextChar();
					//}
					if (_tex_mathstyle != 0)
					{
						_tex_find_math_end( _tex_mathstyle );
								
						/* color the string */
						endoff = _PosQOffset();
						endmathoff = endoff;
						endverbline = _PosQLine();
						endmathline = max( endmathline, endverbline );

						if ( _tex_mathstyle < 10 ) /* Math mode */
						{
				    		_PosInit( startoff );
							_PosSetColor( math_color, endoff - startoff );
						}
						else if (_tex_mathstyle == 10) /* verbatim */
						{
				    		_PosInit( startoff );
							numlines -= (endverbline - _PosQLine()) ;
							_PosSetColor( keyword_color, 6 );
							_PosInit( startoff + 6 );
							_PosSetColor( brace_color, 1 );
							startoff += 7;
							_PosInit( startoff  );
							_PosSetColor( verbatim_color, endoff - startoff);
							_PosInit( startoff + 8 );
							_PosSetColor( brace_color, 1 );
							_PosInit(endoff);
							ch = _PosCurrentChar();
							if ( ch != EOF_CHAR )
							{
								startoff = endoff - 13;
					    		_PosInit( startoff );
								_PosSetColor( keyword_color, 4 );
								_PosInit( startoff + 4 );
								_PosSetColor( brace_color, 1 );
								_PosInit( endoff );
								_PosSetColor( brace_color, 1 );
							}
							startoff = endoff;
							_PosInit(startoff);
							ch = _PosNextChar();
							continue;
						}
						else if (_tex_mathstyle == 11) /* \verb */
						{
				    		_PosInit( startoff );
							numlines -= (endverbline - _PosQLine()) ;
							_PosSetColor( keyword_color, 5 );
							_PosInit( startoff + 5 );
							_PosSetColor( brace_color, 1 );
							startoff += 6;
							_PosInit( startoff  );
							_PosSetColor( verbatim_color, endoff - startoff);
							startoff = endoff;
							_PosInit(startoff);
							ch = _PosCurrentChar();
							if ( ch != EOF_CHAR )
							{
								_PosInit( startoff);
								_PosSetColor( brace_color, 1 );
								_PosInit(startoff);
							}
							ch = _PosNextChar();
							continue;
						}
			    	}
				}
				_PosInit( startoff );
				if ( ch == '\\' )
					goto checkkey;

				ch = _PosNextChar();
				if (ch == '$')
					ch = _PosNextChar();
				continue;
			}
		}

		if (number_color && isdigit( ch ))
		{
            int j = 0;
			UINT lastch = _PosPrevChar();

			if (lastch != EOF_CHAR)
				_PosNextChar();

			startoff = _PosQOffset();
			do
			{
                j++;
				ch = _PosNextChar();
			} while (isdigit( ch ) || (ch == '.') );

			endoff = _PosQOffset();
    		_PosInit( startoff );
			_PosSetColor( number_color, endoff - startoff );
			_PosInit( endoff );
			continue;
		}


		// if "%", the end of the line is commented.
		if (ch == '%' )
		{
			startoff = _PosQOffset();
			//lastch = _PosPrevChar();
			ch = _PosPrevChar();

			if (ch != EOF_CHAR)	// begin of file
				_PosNextChar();
			else if ( lastch == '%' )
				startoff--;

			if (lastch == '\n')
				_PosNextChar();

			if (lastch != '\\' || (lastch == '\\' && ch == '\\'))
			{
				inc = _PosNextLine( 1L );
				if ( inc )
					numlines--;
				else
					_PosNextLine( 0L );
				endoff = _PosQOffset();
				_PosInit( startoff );
				_PosSetColor( comment_color, endoff - startoff - inc );
				_PosInit( endoff );
				ch = _PosCurrentChar();
			}
			else
			{
			ch = _PosNextChar();
			}
			continue;
		}

		else if (brace_color && _fstrchr("{}&", ch) != 0)
		{                                        
			long coff;                            

			coff = _PosQOffset();            
			_PosInit(coff);                  
			_PosSetColor(brace_color, 1);    
			_PosInit(coff);                  
			ch = _PosNextChar();
			continue;
		}                                   

	checkkey:	
		if (ch == '\\')	 //   it's a TeX command
		{
			startoff = _PosQOffset();
			ch = _PosNextChar();
			if (isalpha( ch ) || ch == '@')
			{
				while (isalpha( ch ) || ch == '@')
				{
					ch = _PosNextChar();
				}
			}
			else if (ch != EOF_CHAR) 
			{
				ch = _PosNextChar();
			}
			endoff = _PosQOffset();
			_PosInit( startoff );
			_PosSetColor( keyword_color, endoff - startoff );
			_PosInit( endoff );
			continue;				
		}
		
		ch = _PosNextChar();

	}
	return max(endline, endmathline);
}

/* 
 * TeXColorMath: 	on = 0, disables math coloring
 *				 	on = 1, enables math coloring
 *	 				on = 2, toggles on/off
 */
void DLL
TeXColorMath( int on )
{
	DWORD texflags = TexQFlags();
 	DWORD flags ;
	DWORD flags2;
 	switch (on) 
 	{
 		case 0:
 			flags = (texflags | TEX_COLOR_MATH)  ;
			if (flags != TexSetFlags(flags))
				MsgNotify(" TexSetFlags error ");

 			MsgMessage("Math coloring is off");
 			break;
 
 		case 1:
 			flags = (texflags & ~TEX_COLOR_MATH)  ;
			if (flags != TexSetFlags(flags))
				MsgNotify(" TexSetFlags error ");
 			MsgMessage("Math coloring is on");
 			break;
 
 		case 2:
 			if ( texflags & TEX_COLOR_MATH )
 			{
	 			flags = (texflags & ~TEX_COLOR_MATH)  ;
				flags2 = TexSetFlags(flags);
				if (flags != flags2)
				{
					MsgNotify(" TexSetFlags error ");
				}
 				MsgMessage("Math coloring is on");
 			}
 			else
 			{
	 			flags = (texflags | TEX_COLOR_MATH)  ;
				if (flags != TexSetFlags(flags))
					MsgNotify(" TexSetFlags error ");
 				MsgMessage("Math coloring is off");
 			}
 			break;
 	}
	CheckMenuItem( MenuQHandle() , IDM_TEX_MATHCOLOR,
				( TexQFlags() & TEX_COLOR_MATH ? MF_UNCHECKED : MF_CHECKED ));
 	SysSetSystemFlags( SYSTEM_DELAY_ATTR_CHANGES );
 	AttrSetColor( 1L, 0x7FFFFFFFL, 1L, 0x7FFFFFFFL, 0 );
	_tex_language_colors( -1 );
	/* Allow changes to be redraw 
	 */
	SysClrSystemFlags( SYSTEM_DELAY_ATTR_CHANGES );
	WinRedraw( WinQCurrentWindow() );
	return; 
}


void DLL
_tex_add_keyword( LPSTR keyword )
{
	if (keyword && *keyword)
	{
		LPSTR XFAR *lpf = (LPSTR XFAR *)HashGetEntry( _tex_hash_names, 
						keyword, strlen( keyword ) );
		if (lpf)
			*lpf = StrNew( keyword );
	}
}


/*
 ** _tex_assign_template()
 *
 *  PARAMETERS:
 *    abbrev   - the keyword abreviation used to locate a template
 *    template - the encoded template string associated with 'abbrev'
 *
 *  DESCRIPTION:
 *    Provide a means of adding or modifying TeX template strings
 *    without changing the code.
 */
void DLL
_tex_assign_template( LPSTR abbrev, LPSTR template )
{
	_ext_assign_template( _tex_hash_templates, abbrev, template );
}


/*
 ** _tex_indent()
 *
 *  DESCRIPTION:
 *    Attempts to perform TeX language smart indenting.  This function
 *    will behave differently based on the following conditions:
 *
 *    1. Within a comment - normal indentation
 *
 *    2. Not positioned at end of line - normal indentation
 *
 *    3. Current line prefixed by a name in TEX_INDENT_STR - perform smart
 *       indentation.  This may include skipping the next line if
 *       it consists only of an open brace.
 */
BOOL DLL
_tex_indent( void )
{
	long   curline;
	long   col;
	LPSTR  _tex_string;
	LPSTR  str;
	long   matchpos;
	WORD   endch;

	MarkSavePos();

	curline = BufQCurrentLine();

	/* Check to see if we are currently positioned inside a TeX comment
	 */
	if (_tex_in_comment( -1L ) != -1)
		goto _tex_indent_incomplete;

	/* Check to see if there is a trailing '{' on the current line,
	 * and if so, indent on the next line.
	 */
	do {
		MovPrevChar( 1L );
		endch = BufReadChar();
	} while (endch == ' ' || endch == '\t');
	MarkRestorePos();
	if (endch == '{')
		goto _tex_indent_next_line;
	MarkSavePos();

	/* Not in a comment so indent according to the current TeX construct.
	 */
	if (!SrchFind( "^[ \t]*[a-zA-Z_]+\\c([ \t]|$)", SEARCH_REGEX , NIL )
			|| (BufQCurrentLine() != curline) )
		goto _tex_indent_incomplete;

	/* Read the keyword at the beginning of the current line.
	 */
	col = BufQCurrentCol();
	SrchFind( "(^|[^a-zA-Z_])\\c[a-zA-Z_]", SEARCH_REGEX , NIL );
	str = BufReadStr( col - BufQCurrentCol() );
	_tex_string = StrTrim( str, " \t" );
	StrFree( str );

	/* Set prefix and suffix to \377 to perform a quick lookup of the name
	 */
	str = StrNew( "\377" );
	_tex_string = StrAppend( _tex_string, str );
	str = StrAppend( str, _tex_string );
	StrFree( _tex_string );
	
	matchpos = StrMatch( str, TEX_INDENT_STR, SEARCH_IGCASE | SEARCH_FORWARD, NIL );
	StrFree( str );

	if (!matchpos)
		/* Not a TeX keyword, do normal indentation. */
		goto _tex_indent_incomplete;
		
	MarkRestorePos();
	MarkSavePos();

	/* Check to see if a '{' is by itself on the next line. If so, skip
	 * it and indent on the line following it.
	 */
	if (SrchFind( "^[ \t]*{\\c[ \t]*$", SEARCH_FORWARD | SEARCH_REGEX, NIL )
			&& (BufQCurrentLine() == curline + 1) )
		MarkDropPos( 1 );
	else
		MarkRestorePos();

_tex_indent_next_line:
	_ext_insert_indented_EOL();
	BufInsertChar( '\t' );
	return TRUE;

	_tex_indent_incomplete:
		MarkRestorePos();

	return FALSE;
}




/*--  Private functions --*/


/*
 ** _tex_in_comment()
 *
 *  Determines whether or not the current position in the buffer is
 *  located within a TeX comment by search backward for a % 
 *  comment declaration.
 *
 *  Returns -1 if not in a comment 
 *          >= 0 indicates the offset in the buffer where the comment begins
 */
static long
_tex_in_comment( long offset )
{
	WORD  ch;
	WORD  lastch;
	BOOL  firstline = TRUE;
	//long  maxlines = ExtCommentSearchLimit( -1 );
	//long  col = 1;

	/* Check to see if we are currently positioned inside a TeX comment
	 */
	_PosInit( offset );
	ch = _PosCurrentChar();

	do
	{
		switch( ch )
		{
		case '%':
			ch = _PosPrevChar();
			lastch = _PosPrevChar();
			if (lastch != EOF_CHAR) 
				_PosNextChar();

			if (ch != '\\' || (ch == '\\' && lastch == '\\'))
			{
				if (firstline)
					return _PosQOffset() ;  /* currrent line is in a comment */
				else
					return -1L;    /* Not in a comment */
			}
			break;

		case '\n':
			firstline = FALSE;

		default:
			ch = _PosPrevChar();
			break;
		}
		//col = 0;
	} while ((ch != EOF_CHAR) && firstline);

	return -1L;
}

/*
 ** _tex_find_beg_firstline( long firstline)
 *
 *  Returns the offset of the begining of firstline
 *  
 */
static long 
_tex_find_beg_firstline( long firstline )
{
	long startoff;

	_PosInit( 0 );
	if (firstline == 1)
		_PosPrevLine( 0L );
	else
	{
		_PosNextLine( firstline - 1 );
		_PosPrevLine( 0L );
	}
			
	return startoff = _PosQOffset();
}

/*
 ** _tex_find_comment_start()
 *
 *  Search backwards for a TeX comment sequence. Return 1 if <%> found
 *  and 0 otherwise. Leave the current position
 *  at the beginning of the found comment.
 */

/* No longer used. Commented out.

static long 
_tex_find_comment_start( void )
{
	WORD  ch = _PosCurrentChar();
	WORD  lastch;
	long  lines = 0;
	//long  maxlines = ExtCommentSearchLimit( -1 );

	do
	{
		if (ch == '%')
		{
			ch = _PosPrevChar();
			lastch = _PosPrevChar();
			if(lastch != EOF_CHAR) _PosNextChar();

			if (ch != '\\' || (ch == '\\' && lastch == '\\'))
				_PosNextChar();
				return 1;	 // comment start
		}
		else if (ch == '\n')
		{
				lines++;
				_PosNextChar();
		}
		else 
		{
			ch = _PosPrevChar();
		}
	} while ((ch != EOF_CHAR) && (lines == 0));

	return 0;
}
 */

/*
 ** _tex_find_math_start (startoff )
 *
 *	PARAMETERS: long startoff (offset of start of line to check)
 *
 *  Search backward for a text color attribute. 
 *  Return 1 if one found and, 0 otherwise.  
 */

static long
_tex_find_math_start( long startoff )
{
	//WORD  ch  ;
	UINT  text_color  = ColorText( -1 );
	//UINT  math_color  = ColorAlternate4( -1 );
	UINT  color  ;
	WORD  ch ;

	_PosInit(--startoff);
	color = _PosQColor();
	ch = _PosCurrentChar();


	while( startoff &&  color != text_color && (color + 0xf0) != text_color
			&& ch != EOF_CHAR )
	{
		_PosInit(--startoff);
		color = _PosQColor();
		ch != _PosCurrentChar();
	}
	return startoff;
}


/*
 ** _tex_find_math_inline ( )
 *
 *	PARAMETERS: long startoff (offset of start of line to check)
 *
 *  Search forwards for a TeX math color attribute. 
 *  Return 1 if one found and, 0 otherwise.  
 */

/* No longer used : commented out

static int
_tex_find_math_inline( long startoff )
{
	WORD  ch  ;
	long  currentoff ;
	int count;
	UINT  math_color  = ColorAlternate4( -1 );

	_PosInit(startoff);
	ch = _PosPrevChar();
	if (startoff && _PosQColor() == math_color )
		return 1; 
	ch = _PosNextChar();
	count = 0;
	
	while (	!( ch == '\n' || ch == EOL1_CHAR || ch == EOL2_CHAR || ch == EOF_CHAR )
	          && count < 200 ) // 
	{
		currentoff = _PosQOffset() ;
		count++;
 //MsgNotify ( StrAppend( StrAscii(ch), StrItoA(currentoff,10)) );
		if (_PosQColor() == math_color )
		{
 //MsgNotify ("Math code found");
			return 1;
		}
		_PosInit( currentoff );
		ch = _PosNextChar();
	}
 //MsgNotify ("No math code found");
	return 0;

	//}
}
 
 end unused function  */
  
 

/*
 ** _tex_find_math_end( int mathstyle)
 *
 *  Search forwards for a TeX math end sequence. Return mathstyle 
 *  if one found and, 0 otherwise.  
 *  Leave the current position at the end of the math end sequence.
 *  _tex_mathstyle = 	0 : not math mode
 *						1 : $ ... $
 *						2 : $$ ... $$
 *						3 : \[ ... \]
 *						4 : \( ... \)
 *						5 : \begin{equation} ... \end{equation}
 *						6 : \begin{eqnarray} ... \end{eqnarray}
 *						7 : \begin{eqnarray*} ... \end{eqnarray*}
 *
 *						10 : \begin{verbatim} ... \end{verbatim}
 */
static int
_tex_find_math_end( int mathstyle )
{
	WORD  ch  = _PosCurrentChar();
	WORD  lastch;
	WORD  prevlastch;

 //	if (ch == '%' && mathstyle && mathstyle < 10 )
 //	{
 //		_PosNextLine(0L);
 //		ch = _PosCurrentChar();
 //	}

	while (  _tex_check_math_end( mathstyle ) == 0  )
	{
		prevlastch = _PosPrevChar();

		if(prevlastch != EOF_CHAR)
			lastch = _PosNextChar();
		else
			lastch = _PosCurrentChar();

		ch = _PosNextChar();
		if (ch == EOF_CHAR)
			return mathstyle;
		if ( (ch == '%') && (lastch != '\\' || 
				(lastch == '\\' &&	prevlastch == '\\') ) && (mathstyle < 10) )
		{
			do 
			{
			_PosNextLine(0L);
			lastch = _PosCurrentChar();
			ch = _PosNextChar();
			} while (ch == '%' );
		}
				
	}
	return 0;

	/*}*/
}


/*
 ** _tex_check_math_end( int mathstyle)
 *
 *  Checks if we are at an end math sequence. Return mathstyle if true
 *  and 0 otherwise.  Leave the current position
 *  at the end of the math end sequence.
 *  _tex_mathstyle = 	0 : not math mode
 *						1 : $ ... $
 *						2 : $$ ... $$
 *						3 : \[ ... \]
 *						3 : \( ... \)
 *						5 : \begin{equation} ... \end{equation}
 *						6 : \begin{eqnarray} ... \end{eqnarray}
 *						7 : \begin{eqnarray*} ... \end{eqnarray*}
 *
 *						10 : \begin{verbatim} ... \end{verbatim}
 *						10+ch : \verb(ch) ... (ch)
 */
static int
_tex_check_math_end( int mathstyle )
{
	WORD  prevlastch  	= '@';
	WORD  lastch  	;
	WORD  ch 		;
	WORD  nextch 		= '@';
	long  count;
	long  offset;

	lastch = _PosCurrentChar();
	if (lastch != EOF_CHAR)
	{
		prevlastch = _PosPrevChar();
		_PosNextChar();
	}
	ch = _PosNextChar();
	if (ch != EOF_CHAR)
	{
		nextch = _PosNextChar();
		_PosPrevChar();
	}


	switch ( mathstyle )
	{
		case 1:

			if ( (prevlastch != '$') && (prevlastch != '\\') 
					&& (lastch == '$') && (ch != '$') ) 
			{
				_PosPrevChar(); 
				return 1;
			}
			if 	((prevlastch == '\\')  
			 		&& (ch == '$') && (nextch != '$' ) 	)
				return 1;
			
			break;

		case 2:

			if ( (prevlastch != '\\') && 
				(lastch == '$') && (ch == '$')  )
				return 2;
			break;

		case 3:

			if ( (prevlastch != '\\') &&
				(lastch == '\\') && (ch == ']') )
				return 3;
			break;

		case 4:

			if ( (prevlastch != '\\') &&
				(lastch == '\\') && (ch == ')') )
				return 4;
			break;

		case 5:
		case 6:
		case 7:
		case 10:

			if ( (prevlastch == '\\') ||
				(ch != 'e') || (lastch != '\\') )
				break;
			ch = _PosNextChar();
			if (ch != 'n')
				break;
			ch = _PosNextChar();
			if (ch != 'd')
				break;
			ch = _PosNextChar();
			while ( (ch == ' ') || (ch == '\t') || (ch == '\n') )
				ch = _PosNextChar();
			if (ch != '{')
				break;
			ch = _PosNextChar();
			while ( (ch == ' ') || (ch == '\t') || (ch == '\n') )
				ch = _PosNextChar();
			switch ( ch )
			{
				case 'e':
					if ( (mathstyle == 10) )
						break;
					ch = _PosNextChar();
					if (ch != 'q')
						break;
					ch = _PosNextChar();
					switch (ch)
					{
						case 'u':
							if ( (mathstyle != 5) )
								break; 
							ch = _PosNextChar();
							if (ch != 'a')
								break;
							ch = _PosNextChar();
							if (ch != 't')
								break;
							ch = _PosNextChar();
							if (ch != 'i')
								break;
							ch = _PosNextChar();
							if (ch != 'o')
								break;
							ch = _PosNextChar();
							if (ch != 'n')
								break;
							ch = _PosNextChar();
							while ( (ch == ' ') || (ch == '\t') || (ch == '\n') )
								ch = _PosNextChar();
							if (ch == /*{*/ '}')
								return 5;
							break;

						case 'n':
							if ( (mathstyle != 6) && (mathstyle != 7 ))
								break; 
							ch = _PosNextChar();
							if (ch != 'a')
								break;
							ch = _PosNextChar();
							if (ch != 'r')
								break;
							ch = _PosNextChar();
							if (ch != 'r')
								break;
							ch = _PosNextChar();
							if (ch != 'a')
								break;
							ch = _PosNextChar();
							if (ch != 'y')
								break;
							ch = _PosNextChar();
							if (ch == '*')
							{
								if (mathstyle != 7)
									break;
								ch = _PosNextChar();
								while ( (ch == ' ') || (ch == '\t') || (ch == '\n') )
									ch = _PosNextChar();
								if (ch == /*{*/ '}')
									return 7;
							 }
							 else
							 {
							 	if (mathstyle != 6)
							 		break;	
								while ( (ch == ' ') || (ch == '\t') || (ch == '\n') )
									ch = _PosNextChar();
								if (ch == /*{*/ '}')
									return 6;
							 }
							 break;
					}
					break;

				case 'v':
					if (mathstyle != 10)
						break;
		 			ch = _PosNextChar();
		 			if (ch != 'e')
		 				break;
		 			ch = _PosNextChar();
		 			if (ch != 'r')
		 				break;
		 			ch = _PosNextChar();
		 			if (ch != 'b')
		 				break;
		 			ch = _PosNextChar();
		 			if (ch != 'a')
		 				break;
		 			ch = _PosNextChar();
		 			if (ch != 't')
		 				break;
		 			ch = _PosNextChar();
		 			if (ch != 'i')
		 				break;
		 			ch = _PosNextChar();
		 			if (ch != 'm')
		 				break;
		 			ch = _PosNextChar();
		 			while ( (ch == ' ') || (ch == '\t') || (ch == '\n') )
		 				ch = _PosNextChar();
		 			if (ch == /*{*/ '}')
		 				return 10;
		 			break;
			}
			break;

		case 11: /* \verb(verbch) ... (verbch) */

			offset = _PosQOffset();
			count = 0;
			while ( (ch != verbch ) && (ch != EOF_CHAR ) && (ch != '\n') 
				&& count < 20 )
		
			{
				_PosInit(offset++);
				ch = _PosCurrentChar();
				//MsgNotify( StrAppend(StrItoA(count++,10),StrItoA(offset,10)) );
			}
			return 11;
			
	}

	_PosPrevChar();
	return 0;
}		


/*
 ** _tex_check_math_start( void)
 *
 *  Checks if we are at a begin math sequence. Return mathstyle if
 *  one is found and, 0 otherwise.  Leave the current position
 *  at the end of the begin math sequence.
 *  _tex_mathstyle = 	0 : not math mode
 *						1 : $ ... $
 *						2 : $$ ... $$
 *						3 : \[ ... \]
 *						4 : \( ... \)
 *						5 : \begin{equation} ... \end{equation}
 *						6 : \begin{eqnarray} ... \end{eqnarray}
 *						7 : \begin{eqnarray*} ... \end{eqnarray*}
 *
 *						10 : \begin{verbatim} ... \end{verbatim}
 *						10+ch : \verb(ch) ... (ch)
 */
static int
_tex_check_math_start( void )
{
	WORD  ch  = _PosCurrentChar();


	switch ( ch )
	{
		case '$':

			ch = _PosNextChar();
			if (ch == '$') 
				return 2;
			return 1;

		case '\\':

			ch = _PosNextChar();
			if (ch == '[')
				return 3;
			else if (ch == '(')
				return 4;
		switch (ch )
		{
				//else if (ch != 'b')
				//	break;
			case 'b':
				ch = _PosNextChar();
				if (ch != 'e')
					break;
				ch = _PosNextChar();
				if (ch != 'g')
					break;
				ch = _PosNextChar();
				if (ch != 'i')
					break;
				ch = _PosNextChar();
				if (ch != 'n')
					break;
				ch = _PosNextChar();
				while ( (ch == ' ') || (ch == '\t') || (ch == '\n') )
					ch = _PosNextChar();
				if (ch != '{')
					break;
				ch = _PosNextChar();
				while ( (ch == ' ') || (ch == '\t') || (ch == '\n') )
					ch = _PosNextChar();
				switch ( ch )
				{
					case 'e':
						ch = _PosNextChar();
						if (ch != 'q')
							break;
						ch = _PosNextChar();
						switch (ch )
						{
							case 'u':
								ch = _PosNextChar();
								if (ch != 'a')
									break;
								ch = _PosNextChar();
								if (ch != 't')
									break;
								ch = _PosNextChar();
								if (ch != 'i')
									break;
								ch = _PosNextChar();
								if (ch != 'o')
									break;
								ch = _PosNextChar();
								if (ch != 'n')
									break;
								ch = _PosNextChar();
								while ( (ch == ' ') || (ch == '\t') || (ch == '\n') )
									ch = _PosNextChar();
								if (ch == /*{*/ '}')
									return 5;
								break;

							case 'n':
								ch = _PosNextChar();
								if (ch != 'a')
									break;
								ch = _PosNextChar();
								if (ch != 'r')
									break;
								ch = _PosNextChar();
								if (ch != 'r')
									break;
								ch = _PosNextChar();
								if (ch != 'a')
									break;
								ch = _PosNextChar();
								if (ch != 'y')
									break;
								ch = _PosNextChar();
								if (ch == '*')
								{
									ch = _PosNextChar();
									while ( (ch == ' ') || (ch == '\t') || (ch == '\n') )
										ch = _PosNextChar();
									if (ch == /*{*/ '}')
										return 7;
									break;
								 }
								 else
								 {	
									while ( (ch == ' ') || (ch == '\t') || (ch == '\n') )
										ch = _PosNextChar();
									if (ch == /*{*/ '}')
										return 6;
									break;
								 }
						}
						break;

					case 'v':
			 			ch = _PosNextChar();
			 			if (ch != 'e')
			 				break;
			 			ch = _PosNextChar();
			 			if (ch != 'r')
			 				break;
			 			ch = _PosNextChar();
			 			if (ch != 'b')
			 				break;
			 			ch = _PosNextChar();
			 			if (ch != 'a')
			 				break;
			 			ch = _PosNextChar();
			 			if (ch != 't')
			 				break;
			 			ch = _PosNextChar();
			 			if (ch != 'i')
			 				break;
			 			ch = _PosNextChar();
			 			if (ch != 'm')
			 				break;
			 			ch = _PosNextChar();
			 			while ( (ch == ' ') || (ch == '\t') || (ch == '\n') )
			 				ch = _PosNextChar();
			 			if (ch == /*{*/ '}')
			 				return 10;
			 			break;
				}
				break;

			case 'v':
	 			ch = _PosNextChar();
	 			if (ch != 'e')
	 				break;
	 			ch = _PosNextChar();
	 			if (ch != 'r')
	 				break;
	 			ch = _PosNextChar();
	 			if (ch != 'b')
	 				break;
				ch = _PosNextChar();
					//MsgNotify(StrAppend(StrNew( &ch ),StrItoA((int)ch,10)) );
					verbch = ch;
					return 11 ;
				break;
		}
		
	}
	return 0;
}		

  
/*******************************************************************
 * _tex_routines()
 *******************************************************************/

void DLL
_tex_routines( void )
{
	if (Grep("\\\\begin|\\\\end|\\\\part|\\\\chapter|\\\\subparagragh|\\\\paragraph|\\\\section|\\\\subsection|\\\\subsubsection|\\\\usepackage"))
		MsgMessage("");
	else
		MsgMessage("No routines found.");
}

