/*******************************************************************
 *
 *    DESCRIPTION: TeX error parser (_TeXErrorInfo) 
 *				   Part of TeX support for CodeWRight (TEX.DLL).
 *
 *    AUTHOR: 	Gilbert Ritschard, Department of Econometrics
 *				University of Geneva, CH-1211 Geneva 4
 *				e-mail: ritschar@uni2a.unige.ch
 *
 *    HISTORY:  1.0  6 september, 1994 (for VTeX)
 *           :  2.0  3 July, 1998 (adapted for MiKTeX and use of main file)
 *
 *******************************************************************/

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "exports.h"
#include "cwstart.h"
#include "tex.h" 

LPSTR DLL DecodeSrcName( LPSTR );
BOOL tex_find_matching_brace( void );
void tex_SrcFileName( LPSTR, BOOL );
BOOL srcFileNameFound;
 	
MarkID curr_error;

/*
 ** _TeXErrorInfo
 *
 *  This is the error info function to process error generated by TeX
 *  
 *
 *  The format of the error is:
 *    "\input <filename>  	<-  only once
 *
 *    "l.<line>.<col>(<filename>) <line containing error>"
 *
 *	  " (<filename>
 *     l.<line> <line containing error>
 *	   ...
 *     )"
 *
 *	  "Warning ... "
 *	  "..erfull  ... " (for Overfull and Underfull)
 *	  "Emergency ... "
 *
 *	The error message is found above the error line. It
 *  starts with "!" in the first column.
 *
 */
int DLL
_TeXErrorInfo( void )
{
	LPSTR srchStr;
	//LONG  pos;
	//LONG  len;
	LPSTR msg;
	//LPSTR srcName;
	//DWORD curr_offset, fname_offset;

	switch(tex_warning_level)
	{
		case 0 :
			srchStr = StrNew("^l\\.\\c|Warning|^[UndOv]+erfull|^!? ?Emergency|^Fatal|^Output written|^Missing character:|! File ended while");
			break;

		case 1 :
			srchStr = StrNew("^l\\.\\c|^.?.?.?.?Warning|[^F]... Warning|^[UndOv]+erfull|^!? ?Emergency|^Fatal|^Output written|^Missing character:|! File ended while");
			break;

		case 2 :
			srchStr = StrNew("^l\\.\\c|[^F]... Warning|^[UndOv]+erfull|^!? ?Emergency|^Fatal|^Output written|^Missing character:|! File ended while");
			break;

		case 3 :
			srchStr = StrNew("^l\\.\\c|^[UndOv]+erfull|^!? ?Emergency|^Fatal|^Output written|^Missing character:|! File ended while");
			break;

		case 4 :
			srchStr = StrNew("^l\\.\\c|^!? ?Emergency|^Fatal|^Output written|^Missing character:|! File ended while");
			break;

		default :
			srchStr = StrNew("^l\\.\\c|^Emergency|^Fatal|^Output written|! File ended while");
			break;
	 }


	/* Search through the error file for the next matching error message
	 * leaving the cursor located over the error's line number.
	 */
	
	if (!SrchFind( srchStr, SEARCH_FORWARD | SEARCH_REGEX , NIL ))
			return 0;

	/* Save the position of the line number and make a copy of the 
	 * message line.
	 */
	curr_error = MarkAllocLocal();
	MarkSavePos();
	MovHome();
	msg = BufReadStr( 0x7FFFFFFF );
	MarkRestorePos();
	
	/* If the line begins with 'l.', it's an error
	 * else it is a warning, overfull, etc.
	 */
	ErrorColumn( 0L );
	if(!StrMatch( "^l\\.", msg, 
		SEARCH_FORWARD | SEARCH_REGEX, NULL ))
	{
		tex_SrcFileName( msg, FALSE );

		/* We are at the curr_error mark. 
		 * Reread msg that has been freed by tex_SrcFileName
		 *
		 */
		MovHome();
		ErrorMsgLine( msg = BufReadStr( 0x7FFFFFFF ) );
		if (StrMatch( "lines? [0-9]", msg,
			SEARCH_FORWARD | SEARCH_REGEX | SEARCH_IGCASE, NIL ) > 1)
		{
			if( msg != NIL ) StrFree( msg );
			SrchFind( "lines? \\c", 
				SEARCH_FORWARD | SEARCH_REGEX | SEARCH_IGCASE, NIL );
			ErrorLine( atol( msg = BufReadStr( 12 ) ) );
		}
		else if (StrMatch( "^Output written", msg,
			SEARCH_FORWARD | SEARCH_REGEX , NIL ) > 0) 
		{
			/* MsgNotify( "'Output written' found" );*/
 			ErrorLine( MarkQLine( current_tex_pos ) );
 			ErrorColumn( MarkQColumn( current_tex_pos ) );
			ErrorSrcFile( DecodeSrcName( current_tex_file ) ) ;
		}
		else if(MovDown(1))
		{
			if( msg != NIL ) StrFree( msg );
			MovHome();
			msg = BufReadStr( 0x7FFFFFFF );
			if (StrMatch( "lines? [0-9]", msg,
				SEARCH_FORWARD | SEARCH_REGEX | SEARCH_IGCASE, NIL ) > 1)
			{
				if( msg != NIL ) StrFree( msg );
				SrchFind( "lines? \\c",
					SEARCH_FORWARD | SEARCH_REGEX | SEARCH_IGCASE, NIL );
				ErrorLine( atol( msg = BufReadStr( 12 ) ) );
			}
			else
			{
 				MovUp(1);
				if (srcFileNameFound)
				{
				ErrorLine( 0L );
				}
				else
				{
	 			ErrorLine( MarkQLine( current_tex_pos ) );
	 			ErrorColumn( MarkQColumn( current_tex_pos ) );
				ErrorSrcFile( DecodeSrcName( current_tex_file ) ) ;
				}
			}
		}
		else
		{
			if (srcFileNameFound)
			{
			ErrorLine( 0L );
			}
			else
			{
 			ErrorLine( MarkQLine( current_tex_pos ) );
 			ErrorColumn( MarkQColumn( current_tex_pos ) );
			ErrorSrcFile( DecodeSrcName( current_tex_file ) ) ;
			}
		}

		//MarkGoto(curr_error);
		if( msg != NIL ) StrFree( msg );
		if(MovDown(1))
			MovHome();
		else
			MovEOL();
		return 1;
	}
		
	/* The current position is on the error line number.  Read the
	 * text into a string and convert it to a long integer.
	 */
	ErrorLine( atol( msg = BufReadStr( 12 )) );
	if( msg != NIL ) StrFree( msg );
	/*ErrorColumn( 0L );*/

	/* Strip out the ErrorColumn from the error line.
	 */
	SrchFind( "\\.\\c|\\c\\n", SEARCH_FORWARD | SEARCH_REGEX, NULL );
	ErrorColumn( atol( msg = BufReadStr( 12 )) );
	if( msg != NIL ) StrFree( msg );

	/* Save the position of the line number and make a copy of the 
	 * message line.
	 */
	MarkSavePos();
	MovHome();
	msg = BufReadStr( 0x7FFFFFFF );
	MarkRestorePos();

	tex_SrcFileName( msg, TRUE );

	/* Determine the error message text.  In this case, everything
	 * past the previous '!'.
	 */
	MarkGoto(curr_error);
	//MsgNotify(StrItoA(BufQCurrentLine(),10));
	MarkSavePos();
	if ( !SrchFind( "^!",  SEARCH_REGEX | SEARCH_IGCASE, NULL ) )
		{
		msg = BufReadStr( 0x7FFFFFFF );
		}
	else
		{
		MovHome();
		msg = BufReadStr( 0x7FFFFFFF );
		}
	ErrorMsgLine( msg );
	if( msg != NIL ) StrFree( msg );
	MarkRestorePos();
	if(MovDown(1))
		MovHome();
	else
		MovEOL();
	return 2;
}

/* DecodeSrcName( srcname )
 *
 * Changes the "/" of the source name path into "\":
 *
 */
LPSTR DLL
DecodeSrcName( LPSTR srcname )
{
	LPSTR ptr;
	LPSTR result = NIL;
	int   len;
	int   nlen = 0;

	if (srcname != NIL)
	{
		ptr = srcname;
		len = 0;
		while (*ptr)
		{
			nlen++;
			if (*ptr == '/')
			{
				if (len)
					result = StrNAppend( result, srcname, len );

				result = StrAppend( result, "\\" );

				len  = 0;
				ptr += 1;
				srcname = ptr;
			}
			else
			{
				len++;
				ptr++;
			}
		}

		if (len)
			result = StrNAppend( result, srcname, len );
		
		while (nlen > 0)
		{
			ptr--;
			if (*ptr == '.')
			{
				break;
			}
			nlen--;
		}
		if (nlen == 0)
			result = StrAppend( result, ".tex" );

	}

	return result;
}


/*
 ** tex_find_matching_brace()
 *
 *  PARAMETERS:
 *    LPSTR srchStr : the string with the error syntax
 *
 *  DESCRIPTION:
 *    Find the next ')' skipping error messages 
 *
 *  RETURNS:
 *    TRUE if matching brace found
 *    FALSE otherwise
 */
BOOL 
tex_find_matching_brace( void )
{
	int  count = 1;
	int  i;
	UINT ch, ch_b;
	long lastoff;
	LPSTR msg = NIL;
//	MarkID markb;
//	if(!MovLeft(1))
//	{
//		MovUp(1);
//		MovEOL();
//	}

	//while ( SrchFind( "((\\()|(\\)))", SEARCH_ONCE | SEARCH_REGEX | SEARCH_IGCASE, NIL ) && count > 0 )
	_PosInit( -1 );
	ch = _PosPrevCharNL();
	while ( count > 0 )
	{
		if ((ch != '(') && (ch != ')' ))
		{
			if( (ch = _PosPrevCharNL()) == EOF_CHAR )
				break;
			continue;
		}
		//MsgNotify( StrItoA( BufQCurrentLine(), 10 ) );
		lastoff = _PosQOffset();
		MovToOffset( lastoff );
		//markb = MarkAllocLocal();
		MovHome();
		//msg = BufReadStr( 0x7FFFFFFF );

		/* skip the parenthesis found if in an error message 
		 */
		i = 3;
		while( i > 0 )
		{
			_PosInit(-1);
			ch_b = _PosCurrentChar();
			if( ch_b == '!')
			{		
			 	i = -9;
			 	break;
			}
			else if( ch_b == 'l' && _PosNextChar() == '.' )
			{		
			 	_PosPrevChar();
			 	i = -9;
			 	break;
			}
			else if( ch_b == 'O' || ch_b == 'U' )
			{		
				msg = BufReadStr(11);
				if( StrMatch( "Overfull h|Underfull h" , 
						msg , SEARCH_REGEX | SEARCH_FORWARD , NULL ))
				{
					StrFree( msg );
			 		i = -9;
			 		break;
				}
			}

			MovUp(1);
			MovHome();
			i--;
			continue;
		}
		if ( i != -9 )
		{
			//MarkGoto(markb);
			MovToOffset( lastoff );
			if ( ch == ')'  )
				count++;
			else 
				count--;
		}
		_PosInit(-1);
		if( (ch = _PosPrevCharNL()) == EOF_CHAR )
			break;
	}
	_PosNextCharNL();
	
	if ( msg != NIL )
		StrFree( msg );
	if ( count > 0 )
		return FALSE;
	else
		return TRUE;
}

/*	tex_SrcFileName( msg, errline )
 *
 *  PARAMETERS : msg		a string containing the error line
 *  		     errline  	TRUE if an error ("^l\\." found)
 *							FALSE otherwise 
 *
 *  Extracts the source file name.
 *
 */

void
tex_SrcFileName ( LPSTR msg, BOOL errline )
{
	LONG  pos;
	LONG  len = 0L;
	//LONG  outputwritten = 0L;
	LPSTR srcName;
	DWORD curr_offset, fname_offset;

	/* Strip out the source file name .
	 *
	 * If errline=TRUE,
	 * first check if the file name is given
	 *  between parentheses on the error line 
	 *  (works with VTeX but not with MikTeX)
	 */
	if ( errline )
	{
		pos = StrMatch( "^l[0-9\\.]+\\(\\c.+\\)", msg, 
			SEARCH_FORWARD | SEARCH_REGEX, NULL );
		len = StrMatch( "^l[0-9\\.]+\\((.+)\\c\\)", msg, 
			SEARCH_FORWARD | SEARCH_REGEX, NULL );
		len = len - pos;
	}
 //	else
 //	{
 //		outputwritten =  StrMatch( "^Output written", msg,
 //						SEARCH_FORWARD | SEARCH_REGEX , NIL );
 //	}
	
	/*  If len==0, no file name was found. 
	 *  Then search for a closing parenthesis in column 1
	 *  or after a "]"
	 *  and seek the matching opening parenthesis. The
	 *  source filename is after this parenthesis
	 *  (works with MikTeX)
	 */
	if (!len) 
	{
		/* Save the position on the error line
		   and the current offset
		 */
		MarkSavePos();
		fname_offset = 0;
		curr_offset = BufQCurrentLine();

		/* The file name must be before the error line
		 */
		do
		{
			/* Restore the position of the previous find 
			   closing parenthesis or of the error line
			   at start
			 */
			MarkRestorePos();
			MovRight(1);
			if(!SrchFind( "^[ ]?\\)|\\]\\c\\)|\\) \\c\\)", SEARCH_FORWARD | SEARCH_REGEX | SEARCH_IGCASE, NIL ))
			{
				//MarkSavePos();
				fname_offset = 0;
				break;
			}
			/* Save the position of the find closing parenthesis
			 */
			MarkSavePos();
			tex_find_matching_brace();
			fname_offset = BufQCurrentLine();
		}  while (curr_offset <= fname_offset);


		if(fname_offset > 0)
		{
			msg = BufReadStr( 0x7FFFFFFF );
			//MsgNotify( msg );
			//MsgNotify( StrItoA( BufQCurrentLine(), 10 ));
			/* Strip out the source file name from the error line.
			 */
			pos = StrMatch( "\\(\\c", msg, 
				SEARCH_FORWARD | SEARCH_REGEX, NULL );
			len = StrMatch( "[^ \\t\\(]+\\c([ \\t]|$)", msg, 
				SEARCH_FORWARD | SEARCH_REGEX | SEARCH_MAXIMAL, NULL );
			len = len - pos ;
			//MsgNotify(StrItoA(len,10));
		}
	}

	/* If len > 0, then a file name was found.
	 * Strip it out from the msg string.
	 */
	if (len)
	{ 
		srcFileNameFound = TRUE;
		srcName = StrSubStr( msg, (int) pos, (int) len );
	}
	else //if ( outputwritten ) /* select the current file */
	{
		srcFileNameFound = FALSE;
		srcName = StrNew(current_tex_file) ;
	}
 //	else /* Search for main file name */
 //	{
 //		MovTopBuf();
 //		if (SrchFind( "\\\\input \\c", SEARCH_FORWARD | SEARCH_REGEX | SEARCH_IGCASE, NIL ))
 //		{
 //			srcName = BufReadStr( 0x7FFFFFFF ) ;
 //		}
 //	}
	ErrorSrcFile( srcName = DecodeSrcName( srcName )) ;
	StrFree( srcName );
	if( msg != NIL ) StrFree( msg );
	
	MarkGoto(curr_error);
	return;
}