/*******************************************************************
 *
 *    DESCRIPTION: Menus and dialogs 
 *				   Part of TeX support for CodeWRight (TEX.DLL).
 *
 *    AUTHOR: 	Gilbert Ritschard, Department of Econometrics
 *				University of Geneva, CH-1211 Geneva 4
 *				e-mail: Gilbert.Ritschar@themes.unige.ch
 *
 *    HISTORY:  1.0  September 6, 1994
 *              2.0  November 15, 1996
 *				3.0  July 3, 1998 (for CW32 and new error parser)
 *
 *******************************************************************/

/** include files **/
//#include "win_filt.h"
#undef NOMENUS
#include <windows.h>
#include <commdlg.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
//#include <io.h>
#include "exports.h"
//#include "cwdialog.h"
#include <cwhelpid.h>
#define __CWSTART_STRID__
#include "cwstart.h"

LOCAL_MSG_TBL(CWSTART_MSG_TBL_ID);

#include "tex.h"

#define VALID_PARENT(hWnd)          (IsWindow(hWnd) ? hWnd : ((SysQVariant() & VARIANT_CWVC) ? WinQHClient() : WinQHFrame()))
/** local definitions **/
#define MAX_BUF			255
#define MBUFSIZE		512

#define NOVAL			(UINT)(-1)
	
/** external functions **/

/** external data **/

/** callback functions **/
BOOL DLL dlgTexSetupProc( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam );
BOOL DLL dlgFromPage( HWND hWnd );
BOOL DLL dlgFromPageProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL DLL dlgSpellReload( HWND hWnd );
BOOL DLL dlgSpellReloadProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

PUBLIC int execBuildCommand( LPSTR fname, LPSTR cmnd, LPErrorInfo lpErrinfo, 
		DWORD flags );
PUBLIC BOOL DLL	TexWrapEnable( int on );

/** internal functions **/
PRIVATE BOOL dlgBox(HINSTANCE hInst, LPSTR name, HWND hWnd,
		BOOL (DLL *proc)(HWND, UINT, WPARAM, LPARAM));
PRIVATE void updateTexBuildFields( HWND hDlg, LPSTR desc );

UINT DLL _ExecTeX( LPSTR fname, LPSTR command, DWORD flags, LPErrorInfo lpErrinfo );
UINT DLL _ExecPrint( LPSTR fname, LPSTR command, DWORD flags, LPErrorInfo lpErrinfo );
UINT DLL _ExecSpell( LPSTR fname, LPSTR command, DWORD flags, LPErrorInfo lpErrinfo );


PRIVATE void ascii_to_ansi();
PRIVATE void ascii_to_tex();
PRIVATE void ansi_to_ascii();
PRIVATE void ansi_to_tex();
PRIVATE void tex_to_ascii();
PRIVATE void tex_to_ansi();
UINT TexUseMain( void ); 
UINT TexSetMain( void ); 

static LPSTR getNormalizedExtension( LPSTR ext );
static BOOL  isCommandSpecified( LPSTR fname );


/** public data **/

/** private data **/
WORD nFrompage = 1;
WORD nEndpage = 999;
static LPSTR switches = NULL;

BOOL end_eq_total;

/** public functions **/
PUBLIC void TexMenuInsert();

/*
 ** TexMenuInsert
 *
 *  FILENAME: D:\CW32\TEX\TEXMENU.C	   (gr Jan 4, 1998)
 *
 *  PARAMETERS:	none
 *
 *  DESCRIPTION: Setups the TeX menu
 *
 *  RETURNS: none
 *
 */
void 
TexMenuInsert()
{
	//UINT		status;
	//LPSTR  description;
	//DWORD check = 235L;

	MenuInsert( 3, IDM_TEX_TEX, "Te&X", NULL, "test menu" );

	MenuInsertItem( IDM_TEX_TEX, 0, 0, IDM_TEX_MAIN, "Use &main name", "Use project name as main name" );
	MenuInsertItem( IDM_TEX_TEX, 1, 0, IDM_TEX_WORD_WRAP, "TeX Word Wrap", "Enables/Disables special TeX word wrap" );
	MenuInsertItem( IDM_TEX_TEX, 2, 0, IDM_TEX_MATHCOLOR, "TeX &Math Coloring", "Enables/Disables TeX math coloring" );
	MenuInsertItem( IDM_TEX_TEX, 3, MF_SEPARATOR, 0, "", "" );
	MenuInsertItem( IDM_TEX_TEX, 4, 0, IDM_TEX_TEX1, "Te&X 1", "Execute typesetting command 1" );
	MenuInsertItem( IDM_TEX_TEX, 5, 0, IDM_TEX_TEX2, "TeX 2", "Execute typesetting command 2" );
	MenuInsertItem( IDM_TEX_TEX, 6, MF_SEPARATOR, 0, "", "" );
	MenuInsertItem( IDM_TEX_TEX, 7, 0, IDM_TEX_PRINT1, "&View", "Execute print/view 1 command" );
	MenuInsertItem( IDM_TEX_TEX, 8, 0, IDM_TEX_PRINT2, "&Print/View 2", "Execute print/view 2 command" );
	MenuInsertItem( IDM_TEX_TEX, 9, 0, IDM_TEX_PRINT3, "Print/View 3", "Execute print/view 3 command" );
	MenuInsertItem( IDM_TEX_TEX, 10, 0, IDM_TEX_PRINT4, "Print/View 4", "Execute print/view 4 command" );
	MenuInsertItem( IDM_TEX_TEX, 11, MF_SEPARATOR, 0, "", "" );
	MenuInsertItem( IDM_TEX_TEX, 12, 0, IDM_TEX_SPELL1, "Spell &1", "English spellcheck: spell 1 command" );
	MenuInsertItem( IDM_TEX_TEX, 13, 0, IDM_TEX_SPELL2, "Spell &2", "French spellcheck: spell 2 command" );
	MenuInsertItem( IDM_TEX_TEX, 14, MF_POPUP, IDM_TEX_ACCENTS, "Convert &accents", "Convert French accents" );
		MenuInsertItem( IDM_TEX_ACCENTS, 0, 0, IDM_TEX_ACC_ANSI_TEX, "ANSI -> TeX", "" );
		MenuInsertItem( IDM_TEX_ACCENTS, 1, 0, IDM_TEX_ACC_ANSI_ASCII, "ANSI -> OEM", "" );
		MenuInsertItem( IDM_TEX_ACCENTS, 2, 0, IDM_TEX_ACC_TEX_ANSI, "TeX -> ANSI", "" );
		MenuInsertItem( IDM_TEX_ACCENTS, 3, 0, IDM_TEX_ACC_TEX_ASCII, "TeX -> OEM", "" );
		MenuInsertItem( IDM_TEX_ACCENTS, 4, 0, IDM_TEX_ACC_ASCII_ANSI, "OEM -> ANSI", "" );
		MenuInsertItem( IDM_TEX_ACCENTS, 5, 0, IDM_TEX_ACC_ASCII_TEX, "OEM -> TeX", "" );
	MenuInsertItem( IDM_TEX_TEX, 15, MF_POPUP, IDM_TEX_UTIL, "&Other conversions", "Convert text ( {}$&% ) to TeX" );
		MenuInsertItem( IDM_TEX_UTIL, 0, 0, IDM_TEX_UTIL_TEXT_TEX0, "Text without linebreaks to TeX", "Convert ( {}$&%\" ) and wrap all" );
		MenuInsertItem( IDM_TEX_UTIL, 1, 0, IDM_TEX_UTIL_TEXT_TEX1, "Text with linebreaks to TeX", "Convert ( {}$&%\" ) and wrap all" );
		MenuInsertItem( IDM_TEX_UTIL, 2, 0, IDM_TEX_UTIL_TEX_TEXT0, "TeX to Text without linebreaks", "Convert ( {}$&%\" ) and unwrap all" );
		MenuInsertItem( IDM_TEX_UTIL, 3, 0, IDM_TEX_UTIL_TEX_TEXT1, "TeX to Text with linebreaks", "Convert ( {}$&%\" )" );
		MenuInsertItem( IDM_TEX_UTIL, 4, 0, IDM_TEX_UTIL_TAB_TABU, "Tab to TeX", "Convert tab's to '&'" );
		MenuInsertItem( IDM_TEX_UTIL, 5, 0, IDM_TEX_UTIL_TABU_TAB, "TeX to Tab's", "Convert '&' to tab's" );
	MenuInsertItem( IDM_TEX_TEX, 16, MF_SEPARATOR, 0, "", "" );
	MenuInsertItem( IDM_TEX_TEX, 17, 0, IDM_TEX_WRAP_PAR, "&Wrap Paragraph", "TeX wraps the current paragraph" );
	MenuInsertItem( IDM_TEX_TEX, 18, 0, IDM_TEX_WRAP_ALL, "Wrap all", "TeX Wraps all paragraphs" );
	MenuInsertItem( IDM_TEX_TEX, 19, MF_SEPARATOR, 0, "", "" );
	MenuInsertItem( IDM_TEX_TEX, 20, 0, IDM_TEX_SETUP, "&Setup ...", "Setup of TeX commands" );
	MenuInsertItem( IDM_TEX_TEX, 21, 0, IDM_TEX_SETMAIN, "Set main to &current", "Set main to current root name" );

	MenuRegisterHandler( IDM_TEX_TEX1,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_TEX2,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_PRINT1,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_PRINT2,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_PRINT3,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_PRINT4,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_SPELL1,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_SPELL2,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_ACC_ASCII_ANSI,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_ACC_ASCII_TEX,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_ACC_ANSI_ASCII,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_ACC_ANSI_TEX,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_ACC_TEX_ASCII,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_ACC_TEX_ANSI,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_UTIL_TEXT_TEX0,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_UTIL_TEXT_TEX1,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_UTIL_TEX_TEXT0,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_UTIL_TEX_TEXT1,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_UTIL_TAB_TABU,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_UTIL_TABU_TAB,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_WRAP_PAR,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_WRAP_ALL,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_SETUP,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_MAIN,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_WORD_WRAP,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_MATHCOLOR,"TeXCommand");
	MenuRegisterHandler( IDM_TEX_SETMAIN,"TeXCommand");
	
	//check = TexQFlags();
	//MsgNotify( StrItoA(check,10) );
	CheckMenuItem( MenuQHandle() , IDM_TEX_WORD_WRAP, 
				( TexQFlags() & FLAG_TEX_WRAP ? MF_CHECKED : MF_UNCHECKED));
	CheckMenuItem( MenuQHandle() , IDM_TEX_MAIN,
				(WrapEnable(-1) ? MF_CHECKED : MF_UNCHECKED));

}

/*
 *
 * TeX menu handler.  Returns non-zero if processed.
 *
 */
PUBLIC int DLL
TeXCommand(HWND hWnd, UINT menuID)
{

	switch(menuID)
	{
 	case IDM_TEX_MAIN:
		TexUseMain();
		break;
 	case IDM_TEX_WORD_WRAP:
		ToggleTexWrapEnable();
		break;
	case IDM_TEX_MATHCOLOR :
		TeXColorMath(2);
		break;
	case IDM_TEX_TEX1 :
		LibFunctionExec( "ExecTeX1" );
		break;
	case IDM_TEX_TEX2 :
		LibFunctionExec( "ExecTeX2" );
		break;
	case IDM_TEX_PRINT1 :
		LibFunctionExec( "ExecPrint1" );
		break;
	case IDM_TEX_PRINT2 :
		LibFunctionExec( "ExecPrint2" );
		break;
	case IDM_TEX_PRINT3 :
		LibFunctionExec( "ExecPrint3" );
		break;
	case IDM_TEX_PRINT4 :
		LibFunctionExec( "ExecPrint4" );
		break;
	case IDM_TEX_SPELL1 :
		LibFunctionExec( "ExecSpell1" );
		break;
	case IDM_TEX_SPELL2 :
		LibFunctionExec( "ExecSpell2" );
		break;
	case IDM_TEX_ACC_ASCII_ANSI :
		ascii_to_ansi();
		break;
	case IDM_TEX_ACC_ASCII_TEX :
		ascii_to_tex();
		break;
	case IDM_TEX_ACC_ANSI_ASCII :
		ansi_to_ascii();
		break;
	case IDM_TEX_ACC_ANSI_TEX :
		ansi_to_tex();
		break;
	case IDM_TEX_ACC_TEX_ASCII :
		tex_to_ascii();
		break;
	case IDM_TEX_ACC_TEX_ANSI :
		tex_to_ansi();
		break;
	case IDM_TEX_SETUP :
		LibFunctionExec( "DlgTexSetup" );
		break;
	case IDM_TEX_SETMAIN :
		TexSetMain();
		break;
	case IDM_TEX_UTIL_TEXT_TEX0 :
		TextToTeX(0);
		break;
	case IDM_TEX_UTIL_TEXT_TEX1 :
		TextToTeX(1);
		break;
	case IDM_TEX_UTIL_TEX_TEXT0 :
		TeXToText(0);
		break;
	case IDM_TEX_UTIL_TEX_TEXT1 :
		TeXToText(1);
		break;
	case IDM_TEX_UTIL_TAB_TABU :
		TabToTeXTabular();
		break;
	case IDM_TEX_UTIL_TABU_TAB :
		TeXTabularToTab();
		break;
	case IDM_TEX_WRAP_PAR :
		TexWrapParagraph();
		break;
	case IDM_TEX_WRAP_ALL :
		TexWrapAll();
		break;

	default:
		return(FALSE);
	}
	return(TRUE);
}

 /*
 ** TexUseMain
 *
 *
 */
UINT
TexUseMain( void ) 
{
	DWORD flags = TexQFlags();
 //	LPErrorInfo lpErrinfo;
 //	lpErrinfo = _ExtDescriptionEntry( PROJECT_DESC );

	if ( flags	& FLAG_TEX_MAIN)
	{
		TexSetFlags( flags	& ~FLAG_TEX_MAIN );
		//MsgNotify( lpErrinfo->errorFName  );
		//lpErrinfo->errorFName =
		//	StrGSub( "%y", "%r" , lpErrinfo->errorFName,
		//				SEARCH_IGCASE | SEARCH_ONCE | SEARCH_REGEX	);
		//MsgNotify( lpErrinfo->errorFName  );
		MsgMessage("TeX Main disabled");
	}
	else
	{ 
		TexSetFlags( flags	| FLAG_TEX_MAIN );
		//lpErrinfo->errorFName =
		//	StrGSub( "%r", "%y" , lpErrinfo->errorFName,
		//				SEARCH_IGCASE | SEARCH_ONCE | SEARCH_REGEX	);
		MsgMessage("TeX Main enabled");
	}
	CheckMenuItem( MenuQHandle() , IDM_TEX_MAIN, 
				( !(flags & FLAG_TEX_MAIN) ? MF_CHECKED : MF_UNCHECKED));
	return (UINT) 0;
}

 /*
 ** TexSetMain
 *
 *  	lpErrinfo->errorFunc1	:  main root filename
 *
 */
UINT
TexSetMain( void ) 
{
	LPErrorInfo lpErrinfo;
	LPErrorInfo prjlpErrinfo;

	lpErrinfo = _ExtExtensionEntry( EXT_TEX_CMD );
	prjlpErrinfo = _ExtDescriptionEntry( PROJECT_DESC );

	lpErrinfo->errorFunc1 = StrNew( TransformFilename( BufQFilename(), "%r" ) );
	prjlpErrinfo->wrkdir = StrNew( TransformFilename( BufQFilename(), "%v%p" ) );
	return (UINT) 0;
}

 /*
 ** ExecTeX1
 *  	lpErrinfo->cmdCompile
 *
 */
UINT DLL
ExecTeX1( LPSTR fname )
{
	LPErrorInfo lpErrinfo;
	LPSTR		command = NIL;
	UINT		status;

	lpErrinfo = _ExtExtensionEntry( EXT_TEX_CMD );
	//lpErrinfo = _ExtMergeExtErrinfo( ".@TeX", FALSE );

	if (lpErrinfo && lpErrinfo->cmdCompile && *lpErrinfo->cmdCompile)
	{
		command = StrNew(lpErrinfo->cmdCompile);
		status = _ExecTeX( fname, command, lpErrinfo->compileFlags, lpErrinfo );
		return status;
	}
	MsgNotify( "No `TeX1' command assigned." );
	return (UINT)-1;
}

		
 /*
 ** ExecTeX2
 *		lpErrinfo->cmdMake
 *
 */

UINT DLL
ExecTeX2( LPSTR fname )
{
	LPErrorInfo lpErrinfo;
	LPSTR		command = NIL;
	UINT		status;

	lpErrinfo = _ExtExtensionEntry( EXT_TEX_CMD );

	if (lpErrinfo && lpErrinfo->cmdMake && *lpErrinfo->cmdMake)
	{
		command = StrNew(lpErrinfo->cmdMake);
		status = _ExecTeX( fname, command, lpErrinfo->makeFlags, lpErrinfo );
		return status;
	}
	MsgNotify( "No `TeX2' command assigned." );
	return (UINT)-1;
}

 /*
 ** ExecTeX2
 *		lpErrinfo->cmdMake
 *
 */

UINT DLL
_ExecTeX( LPSTR fname, LPSTR command, DWORD flags, LPErrorInfo lpErrinfo )
{
	UINT		status;
	LPSTR		error_file = NIL, tmp = NIL, log_file = NIL;
	LPSTR		main_root = NIL, wrkdir = NIL;
	LPErrorInfo prjlpErrinfo;
	long		pos, len;

	/* We set a bookmark at the current position
	 * and save the current file name.
	 */
	current_tex_pos = MarkCreateGlobal(1111);
	current_tex_file = BufQFilename();

    tex_warning_level = 0;
	if ( pos = StrMatch( "\\[w[0-9]\\]",  command, 
					SEARCH_IGCASE | SEARCH_ONCE | SEARCH_REGEX, &len ) )
	{
		/* extracting the warning level
		 */
		tex_warning_level = atoi(StrNew(StrSubStr( command, pos + 2, 1 )));
		command = StrGSub( "\\[w[0-9]\\]", "" , command,
					SEARCH_ONCE | SEARCH_REGEX	);
	}


	prjlpErrinfo = (LPErrorInfo) _ExtDescriptionEntry( PROJECT_DESC );

	if ( TexQFlags() & FLAG_TEX_MAIN )
	{
		/* If not empty, errorFunc1 is the main root name. 
		 */
		if (lpErrinfo && lpErrinfo->errorFunc1 && *lpErrinfo->errorFunc1)
			main_root = StrNew(	lpErrinfo->errorFunc1 );
		else
		 	main_root = StrNew( "%y" );

		
		/* If main_root contains a path, then set this path as the wrkdir
		 */
		tmp = StrNew( main_root );
		wrkdir = StrNew( TransformFilename( tmp,"%v%p") );
		if ( wrkdir && *wrkdir )
		{
			prjlpErrinfo->wrkdir = StrNew( wrkdir );
			//StrFree( wrkdir );
			main_root =  StrNew( TransformFilename( main_root,"%r" ) ) ;
			lpErrinfo->errorFunc1 = main_root;
		}
		command = StrGSub( "%r", main_root , command,
					SEARCH_ONCE | SEARCH_REGEX	);

		if (prjlpErrinfo && prjlpErrinfo->wrkdir && *prjlpErrinfo->wrkdir)
		{
			wrkdir = StrNew( prjlpErrinfo->wrkdir );
			//MsgNotify(wrkdir);
			MsgMessage(wrkdir);
			SysSetCwd(wrkdir);
		}
		else
		{
			MsgMessage("wrkdir empty or not found");
		}

	}
	else
	{
	 	SysSetCwd(TransformFilename( BufQFilename(),"%v%p"));

		/* If a wrkdir is specified and main_root contains no path
		 * append wrkdir to main_root
		 */
		
		/* If not empty, errorFunc1 is the main root name. 
		 */
		if (lpErrinfo && lpErrinfo->errorFunc1 && *lpErrinfo->errorFunc1)
		{
			main_root = StrNew( lpErrinfo->errorFunc1 );
			tmp = StrNew( main_root );
			wrkdir = TransformFilename( tmp ,"%v%p") ;
		
			if ( !(wrkdir && *wrkdir) )
			{
				wrkdir = StrNew( prjlpErrinfo->wrkdir );
				wrkdir = StrAppend( wrkdir,	main_root) ;
				lpErrinfo->errorFunc1  = wrkdir;
				//MsgNotify(lpErrinfo->errorFunc1);
				//StrFree( wrkdir );
				prjlpErrinfo->wrkdir = "";
			}
		}
	}

	status = execBuildCommand( fname, command, lpErrinfo, flags );
	if ( command && *command )
		StrFree(command);
	if (((int)status != -1) && !(flags & ERRCMD_WIN_BACKGROUND))
	{	
		if (lpErrinfo && lpErrinfo->errorFName && *lpErrinfo->errorFName)
			error_file = StrNew( lpErrinfo->errorFName );
		{
			if( StrMatch( "_err_", error_file, 
				SEARCH_FORWARD | SEARCH_REGEX, NULL ))
			{
				/* We rename the "%r.log" or "main_root.log"
				   as "_err_.log"
				 */

				if ( TexQFlags() & FLAG_TEX_MAIN )
				{
					log_file = StrGSub( "_err_", main_root , error_file,
							SEARCH_ONCE | SEARCH_REGEX	);
				}
				else
				{
					log_file = StrNew( TransformFilename( BufQFilename(),"%r"));
					log_file = StrGSub( "_err_", log_file , error_file,
							SEARCH_ONCE | SEARCH_REGEX	);
				}

				/* Decode log_file name	and error_file
				 */
				tmp = DecodeFName( BufQFilename() , error_file ,
						   DECODE_LOWER | DECODE_PROJECT | DECODE_PERCENT );
				error_file = StrNew(tmp);
				tmp = DecodeFName( BufQFilename() , log_file ,
						   DECODE_LOWER | DECODE_PROJECT | DECODE_PERCENT );
				log_file = StrNew(tmp);
				
				if( !FileExists( log_file ))
				{
					MsgNotify(StrAppend(log_file,StrNew(" Not found ")));
					tmp = StrNew("");
						StrFree( tmp );
					log_file = StrNew("");
						StrFree( log_file );
					error_file = StrNew(""); 
						StrFree(error_file);
					main_root = StrNew("");
						StrFree(main_root);
					wrkdir = StrNew("");
						StrFree(wrkdir);
					return status;
				}


				if( FileExists( error_file )) 
					FileDelete ( error_file );

				tmp = StrNew(TransformFilename( error_file , "%r%e" ));
				error_file = StrNew( tmp );
				log_file = StrAppend( StrNew( "rename " ), log_file);
				log_file = StrAppend( log_file , " " );
				log_file = StrAppend( log_file, error_file );
				ExecUserCmnd( log_file , 1);
			}
			ErrorNext( 0, NIL );
		}
	}
	tmp = StrNew("");
		StrFree( tmp );
	log_file = StrNew("");
		StrFree( log_file );
	error_file = StrNew(""); 
		StrFree(error_file);
	main_root = StrNew("");
		StrFree(main_root);
	wrkdir = StrNew("");
		StrFree(wrkdir);
	return status;
}

 /*
 ** ExecPrint1
 *		lpErrinfo->cmdRebuild
 *
 */

UINT DLL
ExecPrint1( LPSTR fname )
{
	LPErrorInfo lpErrinfo;
	UINT		status;
	LPSTR		command = NIL;

	lpErrinfo = _ExtExtensionEntry( EXT_TEX_CMD );

	if (lpErrinfo && lpErrinfo->cmdRebuild && *lpErrinfo->cmdRebuild)
	{
		command = StrNew( lpErrinfo->cmdRebuild );
		status = _ExecPrint( fname, command, lpErrinfo->rebuildFlags, lpErrinfo );
		return status;
	}
	MsgNotify( "No `Print/View 1' command assigned." );
	return (UINT)-1;
}


 /*
 ** ExecPrint2
 *		lpErrinfo->cmdDebug
 *
 */

UINT DLL
ExecPrint2( LPSTR fname )
{
	LPErrorInfo lpErrinfo;
	UINT		status;
	LPSTR		command = NIL;

	lpErrinfo = _ExtExtensionEntry( EXT_TEX_CMD );

	if (lpErrinfo && lpErrinfo->cmdDebug && *lpErrinfo->cmdDebug)
	{
		command = StrNew( lpErrinfo->cmdDebug );
		status = _ExecPrint( fname, command, lpErrinfo->debugFlags, lpErrinfo );
		return status;
	}
	MsgNotify( "No `Print/View 2' command assigned." );
	return (UINT)-1;
}


 /*
 ** ExecPrint3
 *		lpErrinfo->cmdExec
 *
 */

UINT DLL
ExecPrint3( LPSTR fname )
{
	LPErrorInfo lpErrinfo;
	UINT		status;
	LPSTR		command = NIL;

	lpErrinfo = _ExtExtensionEntry( EXT_TEX_CMD );

	if (lpErrinfo && lpErrinfo->cmdExec && *lpErrinfo->cmdExec)
	{
		command = StrNew( lpErrinfo->cmdExec );
		status = _ExecPrint( fname, command, lpErrinfo->execFlags, lpErrinfo );
		return status;
	}
	MsgNotify( "No `Print/View 3' command assigned." );
	return (UINT)-1;
}


 /*
 ** ExecPrint4
 *		lpErrinfo->cmdCompileDbg
 *
 */
UINT DLL
ExecPrint4( LPSTR fname )
{
	LPErrorInfo lpErrinfo;
	UINT		status;
	LPSTR		command = NIL;
	LPSTR		sw_start = NIL;
	LPSTR		sw_end = NIL;
	LPSTR		sw_npage = NIL;

	lpErrinfo = _ExtExtensionEntry( EXT_TEX_CMD );

	if (lpErrinfo && lpErrinfo->cmdCompileDbg && *lpErrinfo->cmdCompileDbg)
	{
		command = StrNew( lpErrinfo->cmdCompileDbg );
		// we use execFlags for both Print3 and Print4;
		status = _ExecPrint( fname, command, lpErrinfo->execFlags, lpErrinfo );
		return status;
	}
	MsgNotify( "No `Print/View 4' command assigned." );
	return (UINT)-1;
}

 /*
 ** _ExecPrint
 *
 *  common part of ExecPrint1 to ExecPrint4
 *  1. current buffer root (%r) is replaced with selected
 *     main root if FLAG_TEX_MAIN is set in TexFlags.
 *     The main root is ErrFName if not empty or project
 *	   root (%y) if empty.	
 *  2. A dialog popups asking for starting and end page to print.
 *     values are substituted to the [s] and [e] parameters
 *     in the print command. In case [n] is used, it is replaced
 *     with the number of pages (end - start + 1).
 */

UINT DLL
_ExecPrint( LPSTR fname, LPSTR command, DWORD flags, LPErrorInfo lpErrinfo )
{
	UINT		status;
	LPSTR		sw_start = NIL;
	LPSTR		sw_end = NIL;
	LPSTR		sw_npage = NIL;
	long		len ;

	if ( TexQFlags() & FLAG_TEX_MAIN )
	{
		LPSTR	main_root;
		// If not empty, errorFunc1 is the main root name. 
		if (lpErrinfo && lpErrinfo->errorFunc1 && *lpErrinfo->errorFunc1)
			main_root = StrNew(	lpErrinfo->errorFunc1 );
		else
		 	main_root = StrNew( "%y" );
		command = StrGSub( "%r", main_root , command,
					SEARCH_ONCE | SEARCH_REGEX	);
		StrFree( main_root );
	}

	/* If there is "[s]", "[e]" or "[n]" in command
	 * popup the FromPage dialog and pass the
	 * the values to the command string
	 */ 
	if ( StrMatch( "\\[[sen]\\]",  command, 
					SEARCH_IGCASE | SEARCH_ONCE | SEARCH_REGEX, &len ) )
	{
		if ( !(BOOL)dlgFromPage((HWND) ModuleInstance ))
		{
			return (UINT)-1;
		}
		// append switches provided in the FromPage dialog
		command = StrAppend( command, " " );
		command = StrAppend( command, switches );
		// replace the "[s]", "[e]" or "[n]" with the appropriate numbers 
		sw_start = 	StrItoA( nFrompage, 10 );
		sw_end   =  StrItoA( nEndpage ,	10 );
		sw_npage =  StrItoA( nEndpage - nFrompage + 1 ,	10 );
		command = StrGSub( "\\[s\\]", sw_start, command, 
						SEARCH_IGCASE | SEARCH_ONCE | SEARCH_REGEX	);
		command = StrGSub( "\\[e\\]", sw_end  , command, 
						SEARCH_IGCASE | SEARCH_ONCE | SEARCH_REGEX	);
		command = StrGSub( "\\[n\\]", sw_npage, command, 
						SEARCH_IGCASE | SEARCH_ONCE | SEARCH_REGEX	);
		}

	status = execBuildCommand( fname, command, lpErrinfo, flags );
	if (((int)status != -1) && !( flags & ERRCMD_WIN_BACKGROUND))
		ErrorNext( 0, NIL );
	if ( command != NIL )
		StrFree(command);
	if (sw_start != NIL )
		StrFree(sw_start);
	if (sw_end != NIL )
		StrFree(sw_end);
	if (sw_npage != NIL )
		StrFree(sw_npage);
	return status;
}

 /*
 ** ExecSpell1
 *		lpErrinfo->alternate1
 *
 */
UINT DLL
ExecSpell1( LPSTR fname )
{
	LPErrorInfo lpErrinfo;
	UINT		status;
	LPSTR		command = NIL;

	lpErrinfo = _ExtExtensionEntry( EXT_TEX_CMD );

	if (lpErrinfo && lpErrinfo->errorFunc2 && *lpErrinfo->errorFunc2)
	{
		command = StrNew( lpErrinfo->errorFunc2 );
		// we use compileDFlags for both Spell1 and Spell2;
		status = _ExecSpell( fname, command, lpErrinfo->compileDFlags, lpErrinfo );
		return status;
	}
	MsgNotify( "No `Spell 1' command assigned." );
	return (UINT)-1;
}

 /*
 ** ExecSpell2
 *		lpErrinfo->alternate2
 *
 */
UINT DLL
ExecSpell2( LPSTR fname )
{
	LPErrorInfo lpErrinfo;
	UINT		status;
	LPSTR		command = NIL;

	lpErrinfo = _ExtExtensionEntry( EXT_TEX_CMD );

	if (lpErrinfo && lpErrinfo->errorFunc3 && *lpErrinfo->errorFunc3)
	{
		command = StrNew( lpErrinfo->errorFunc3 );
		// we use compileDFlags for both Spell1 and Spell2;
		status = _ExecSpell( fname, command, lpErrinfo->compileDFlags, lpErrinfo );
		return status;
	}
	MsgNotify( "No `Spell 2' command assigned." );
	return (UINT)-1;
}

 /*
 ** _ExecSpell
 *  
 *  common part of ExecSpell1 and ExecSpell2
 *  Current buffer is saved in an tempory file tmp_@_.spl.
 *  A dialog popups up after spell command was runned
 *  giving a chance to retrieve the original content of the buffer.
 *
 */
UINT DLL
_ExecSpell( LPSTR fname, LPSTR command, DWORD flags, LPErrorInfo lpErrinfo )
{
	UINT		status;
	DWORD		sflags;

	BufWriteFile( "tmp_@_.spl", FALSE );
	sflags = SysQFlags();
	SysSetFlags( sflags | SYSFLAG_SUPPRESS_RELOAD_PROMPT );
	status = execBuildCommand( fname, command, lpErrinfo, flags	);
	SysSetFlags( sflags );
	if ( !(BOOL)dlgSpellReload((HWND) ModuleInstance ))
	{
		MsgNotify( "Error in dlgSpellReload" );
		return (UINT)-1;
	}
	FileDelete( "tmp_@_.spl" );
	if ( command != NIL )
		StrFree(command);
	return status;
}

 /** dlgFromPage */

BOOL DLL
dlgFromPage( HWND hWnd )
{
	 if(dlgBox(ModuleInstance, "FROMPAGE", VALID_PARENT(hWnd), dlgFromPageProc))
	 	return TRUE;
	 return FALSE;
}

 /* Ask for starting and end page */

BOOL DLL
dlgFromPageProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	char buf[20+1];

	switch(message)
	{
	case WM_INITDIALOG:

		SetDlgItemText( hDlg, IDD_FRPG_FROMPAGE, StrItoA(nFrompage,10));
		SetDlgItemText( hDlg, IDD_FRPG_TOPAGE, StrItoA(nEndpage,10 ));
		SetDlgItemText( hDlg, IDD_FRPG_PARAM, switches );

	case WM_COMMAND:
		switch (wParam)
		{
		case IDD_FRPG_OK:
		case IDOK:
			GetDlgItemText( hDlg, IDD_FRPG_FROMPAGE, buf, 4);
			nFrompage = atoi(buf);
			GetDlgItemText( hDlg, IDD_FRPG_TOPAGE, buf, 4);
			nEndpage = atoi(buf);
			GetDlgItemText( hDlg, IDD_FRPG_PARAM, buf, 20);
			switches = StrNew(buf);

			EndDialog (hDlg, 1);
			return (TRUE);
			break;

		case IDD_FRPG_CANCEL:
		case IDCANCEL:
			EndDialog (hDlg, 0);
			return (FALSE);
			break;

		default:
			return (FALSE);
		}

     }
	 return( FALSE );
}



 /** dlgSpellReload */

BOOL DLL
dlgSpellReload( HWND hWnd )
{
	 if(dlgBox(ModuleInstance, "TEXRELOAD", VALID_PARENT(hWnd), dlgSpellReloadProc))
	 	return TRUE;
	 return FALSE;
}


BOOL DLL
dlgSpellReloadProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	long 	line;
	long	col;

	switch(message)
	{
	case WM_COMMAND:
		switch (wParam)
		{
		case IDOK:
			
			EndDialog (hDlg, 1);
			return (TRUE);
			break;

		case IDCANCEL:
			
			line = BufQCurrentLine() - 1;
			col = BufQCurrentCol() - 1;
			BufClear();
			BufInsertFile( "tmp_@_.spl" );
			BufWrite();
			MovTopBuf();
			MovRel( line, col );
			EndDialog (hDlg, 1);
			return (TRUE);
			break;

		default:
			return (FALSE);
		}

     }
	 return( FALSE );
}

/*
 ** DlgTexSetup
 *
 * Put up the TexSetup editing dialog.
 *
 */
BOOL DLL
DlgTexSetup(HWND hWndParent)
{
 //	/* invoke the dialog box */
 //	status = TabDialogBoxParam(ModuleInstance, "EXT_SETUP",
 //			VALID_PARENT(hWndParent), dlgExtSetupProc, (long)extensionDlgs);

  	if ( !dlgBox(ModuleInstance, "TEX_SETUP", VALID_PARENT(hWndParent), 
 	 	dlgTexSetupProc))
	return FALSE;
return TRUE;
}

// TEX_SETUP
BOOL DLL
dlgTexSetupProc( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam )
{
	char   buf[ MAX_BUF+1 ];
	BOOL   selchange;
	LPSTR  description;
	int    nbytes;
 	LPErrorInfo lpErrinfo;


	switch (message) 
	{
	case WM_INITDIALOG:
		_PopulateDescriptions( hDlg, IDD_TEX_DESCRIPTION, CB_ADDSTRING, FALSE );

		SetDlgItemText( hDlg, IDD_TEX_DESCRIPTION, 
				_ExtExtensionDescrip( EXT_TEX_CMD ) );

		WM_COMMAND_POST( hDlg, IDD_TEX_DESCRIPTION, 0, CBN_EDITCHANGE );
		return (TRUE);

	case WM_COMMAND:
		switch (WM_COMMAND_ID(wParam)) 
		{

		case IDD_TEX_DESCRIPTION:
			selchange = FALSE;

			switch (WM_COMMAND_NOTIFY(wParam, lParam ))
			{
			case CBN_SELCHANGE:
				selchange = TRUE;
			case CBN_EDITCHANGE:
				nbytes = 0;
				if ( selchange ||
						((nbytes = GetDlgItemText( hDlg, IDD_TEX_DESCRIPTION,
						buf, MAX_BUF )) != 0))
				{
					if (selchange)
					{
						long tmp;

						tmp = SendDlgItemMessage( hDlg, IDD_TEX_DESCRIPTION, CB_GETCURSEL, 0, 0L );
						tmp = SendDlgItemMessage( hDlg,  IDD_TEX_DESCRIPTION, CB_GETLBTEXT, 
								(UINT)tmp, (long)buf );
						if (tmp != CB_ERR)
							nbytes = lstrlen( buf );
					}

					description = StrNNew( buf, nbytes );
					updateTexBuildFields( hDlg, description );
					StrFree( description );
				}
				break;
			}
			break;

		case IDOK:
			{
				//DWORD compFlags  = 0;
				//DWORD compFlags2 = 0;
				LPSTR compile;
				DWORD compileFlags = 0;
				LPSTR make;
				DWORD makeFlags = 0;
				LPSTR rebuild;
				DWORD rebuildFlags = 0;
				LPSTR debug;
				DWORD debugFlags = 0;
				LPSTR exec;
				DWORD execFlags = 0;
				LPSTR compileDbg;
				DWORD compileDbgFlags = 0;
				LPSTR errorFunc1;  
				LPSTR errorFunc2;  //alternate1
				LPSTR errorFunc3;  //alternate2
				LPSTR errorFName;  
				LPSTR empty = "";

				if (GetDlgItemText( hDlg, IDD_TEX_DESCRIPTION, buf, MAX_BUF ))
				{
					description = StrNew( buf );

					if (description && *description)
					{
						
						CompilerAssign( description, EXT_TEX_CMD ) ;

 /*
 *		tex1	:	compile
 *		tex2	:	make
 *		print1	:	rebuild
 *		print2	:	debug
 *		print3	:	exec
 *		print4	:	compileDbg
 *		spell1	:	alternate1 (errorFunc2)
 *		spell2	:	alternate2 (errorFunc3)
 *		startpageswitch  : errorFunc1
 *		endpageswitch	 : errorFNname
 */
						GetDlgItemText( hDlg, IDD_TEX_TEX1, buf, MAX_BUF );
						compile = StrNew( buf );
						GetDlgItemText( hDlg, IDD_TEX_TEX2, buf, MAX_BUF );
						make = StrNew( buf );
						GetDlgItemText( hDlg, IDD_TEX_PRINT1, buf, MAX_BUF );
						rebuild = StrNew( buf );
						GetDlgItemText( hDlg, IDD_TEX_PRINT2, buf, MAX_BUF );
						debug = StrNew( buf );
						GetDlgItemText( hDlg, IDD_TEX_PRINT3, buf, MAX_BUF );
						exec = StrNew( buf );
						GetDlgItemText( hDlg, IDD_TEX_PRINT4, buf, MAX_BUF );
						compileDbg = StrNew( buf );
						GetDlgItemText( hDlg, IDD_TEX_SPELL1, buf, MAX_BUF );
						errorFunc2 = StrNew( buf );
						GetDlgItemText( hDlg, IDD_TEX_SPELL2, buf, MAX_BUF );
						errorFunc3 = StrNew( buf );
						GetDlgItemText( hDlg, IDD_TEX_MAIN, buf, MAX_BUF );
						errorFunc1 = StrNew( buf );
						GetDlgItemText( hDlg, IDD_TEX_ERRFNAME, buf, MAX_BUF );
						errorFName = StrNew( buf );

						if (IsDlgButtonChecked( hDlg, IDD_TEX_TEX1_CAP))
							compileFlags |= ERRCMD_CAPTURE_OUTPUT;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_TEX2_CAP))
							makeFlags |= ERRCMD_CAPTURE_OUTPUT;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT1_CAP))
							rebuildFlags |= ERRCMD_CAPTURE_OUTPUT;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT2_CAP))
							debugFlags |= ERRCMD_CAPTURE_OUTPUT;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT3_CAP))
							execFlags |= ERRCMD_CAPTURE_OUTPUT;
//						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT4_CAP))
//							execFlags |= ERRCMD_CAPTURE_OUTPUT;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_SPELL1_CAP))
							compileDbgFlags |= ERRCMD_CAPTURE_OUTPUT;
//						if (IsDlgButtonChecked( hDlg, IDD_TEX_SPELL2_CAP))
//							compileDbgFlags |= ERRCMD_CAPTURE_OUTPUT;

						if (IsDlgButtonChecked( hDlg, IDD_TEX_TEX1_NC))
							compileFlags |= ERRCMD_NOSHELL;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_TEX2_NC))
							makeFlags |= ERRCMD_NOSHELL;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT1_NC))
							rebuildFlags |= ERRCMD_NOSHELL;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT2_NC))
							debugFlags |= ERRCMD_NOSHELL;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT3_NC))
							execFlags |= ERRCMD_NOSHELL;
//						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT4_NC))
//							execFlags |= ERRCMD_NOSHELL;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_SPELL1_NC))
							compileDbgFlags |= ERRCMD_NOSHELL;
//						if (IsDlgButtonChecked( hDlg, IDD_TEX_SPELL2_NC))
//							compileDbgFlags |= ERRCMD_NOSHELL;

						if (IsDlgButtonChecked( hDlg, IDD_TEX_TEX1_SAF))
							compileFlags |= ERRCMD_SAVEALL;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_TEX2_SAF))
							makeFlags |= ERRCMD_SAVEALL;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT1_SAF))
							rebuildFlags |= ERRCMD_SAVEALL;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT2_SAF))
							debugFlags |= ERRCMD_SAVEALL;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT3_SAF))
							execFlags |= ERRCMD_SAVEALL;
//						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT4_SAF))
//							execFlags |= ERRCMD_SAVEALL;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_SPELL1_SAF))
							compileDbgFlags |= ERRCMD_SAVEALL;
//						if (IsDlgButtonChecked( hDlg, IDD_TEX_SPELL2_SAF))
//							compileDbgFlags |= ERRCMD_SAVEALL;

//						if (IsDlgButtonChecked( hDlg, IDD_TEX_TEX1_VDOS))
//							compileFlags |= ERRCMD_USEVDOS;
//						if (IsDlgButtonChecked( hDlg, IDD_TEX_TEX2_VDOS))
//							makeFlags |= ERRCMD_USEVDOS;
//						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT1_VDOS))
//							rebuildFlags |= ERRCMD_USEVDOS;
//						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT2_VDOS))
//							debugFlags |= ERRCMD_USEVDOS;
//						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT3_VDOS))
//							execFlags |= ERRCMD_USEVDOS;
//						if (IsDlgButtonChecked( hDlg, IDD_TEX_SPELL1_VDOS))
//							compileDbgFlags |= ERRCMD_USEVDOS;

						if (IsDlgButtonChecked( hDlg, IDD_TEX_TEX1_MAX ))
							compileFlags |= ERRCMD_WIN_MAX;
						else if (IsDlgButtonChecked( hDlg, IDD_TEX_TEX1_MIN ))
							compileFlags |= ERRCMD_WIN_MIN;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_TEX2_MAX ))
							makeFlags |= ERRCMD_WIN_MAX;
						else if (IsDlgButtonChecked( hDlg, IDD_TEX_TEX2_MIN ))
							makeFlags |= ERRCMD_WIN_MIN;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT1_MAX ))
							rebuildFlags |= ERRCMD_WIN_MAX;
						else if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT1_MIN ))
							rebuildFlags |= ERRCMD_WIN_MIN;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT2_MAX ))
							debugFlags |= ERRCMD_WIN_MAX;
						else if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT2_MIN ))
							debugFlags |= ERRCMD_WIN_MIN;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT3_MAX ))
							execFlags |= ERRCMD_WIN_MAX;
						else if (IsDlgButtonChecked( hDlg, IDD_TEX_PRINT3_MIN ))
							execFlags |= ERRCMD_WIN_MIN;
						if (IsDlgButtonChecked( hDlg, IDD_TEX_SPELL1_MAX ))
							compileDbgFlags |= ERRCMD_WIN_MAX;
						else if (IsDlgButtonChecked( hDlg, IDD_TEX_SPELL1_MIN ))
							compileDbgFlags |= ERRCMD_WIN_MIN;


//						CompilerAddBuild( description, compFlags, compile, make,
//									rebuild, debug, exec, empty, empty,
//									empty, compileDbg, empty, empty,
//									alternate1, alternate2, compFlags2 );

						CompilerAddCmd( description,
										compile, 
										compileFlags, 
										compileDbg,
										compileDbgFlags,
										make, 
										makeFlags, 
										rebuild, 
										rebuildFlags, 
										debug, 
										debugFlags, 
										exec, 
										execFlags, 
										errorFunc1,
										errorFunc2,
										errorFunc3,
										errorFName,
										empty );

 					lpErrinfo = _ExtDescriptionEntry( PROJECT_DESC );
					lpErrinfo->errorFName = errorFName;


						StrFree( compile );
						StrFree( make );
						StrFree( rebuild );
						StrFree( debug );
						StrFree( exec );
						StrFree( compileDbg );
						StrFree( errorFunc2 );
						StrFree( errorFunc3 );

						if (SysQFlags() & SYSFLAG_CONFIG_UPDATE)
							ConfigUpdate( CFG_UPDATE_COMPILER_SETUP 
								| CFG_UPDATE_COMPILER_EXTENSIONS);
					}
					StrFree( description );
				}
			}
			EndDialog(hDlg, 1);
			break;

		case IDCANCEL:
			EndDialog(hDlg, 0);
			break;

		default:
			return(FALSE);
		}
		return(TRUE);
	}
	return (FALSE);
}

/** private functions **/

  /*
 ** updateTexBuildFields
 *
 *  PARAMETERS:	HWND hDlg, LPSTR desc
 *
 *  DESCRIPTION:
 *		tex1	:	cmdCompile
 *		tex2	:	cmdMake
 *		print1	:	cmdRebuild
 *		print2	:	cmdDebug
 *		print3	:	cmdExec
 *		print4	:	cmdCompileDbg
 *		spell1	:	alternate1 (errorFunc2)
 *		spell2	:	alternate2 (errorFunc3)
 *
 *  RETURNS:
 *
 */

PRIVATE void
updateTexBuildFields( HWND hDlg, LPSTR desc )
{
	LPErrorInfo lpErrinfo;

	SetDlgItemText( hDlg, IDD_TEX_TEX1, "" );
	SetDlgItemText( hDlg, IDD_TEX_TEX2, "" );
	SetDlgItemText( hDlg, IDD_TEX_PRINT1, "" );
	SetDlgItemText( hDlg, IDD_TEX_PRINT2, "" );
	SetDlgItemText( hDlg, IDD_TEX_PRINT3, "" );
	SetDlgItemText( hDlg, IDD_TEX_PRINT4, "" );
	SetDlgItemText( hDlg, IDD_TEX_SPELL1, "" );
	SetDlgItemText( hDlg, IDD_TEX_SPELL2, "" );
	SetDlgItemText( hDlg, IDD_TEX_MAIN, "" );
	SetDlgItemText( hDlg, IDD_TEX_ERRFNAME, "" );

	lpErrinfo = _ExtDescriptionEntry( desc );
	if (lpErrinfo)
	{
		if (lpErrinfo->cmdCompile)
			SetDlgItemText( hDlg, IDD_TEX_TEX1, lpErrinfo->cmdCompile );
		if (lpErrinfo->cmdMake)
			SetDlgItemText( hDlg, IDD_TEX_TEX2, lpErrinfo->cmdMake );
		if (lpErrinfo->cmdRebuild)
			SetDlgItemText( hDlg, IDD_TEX_PRINT1, lpErrinfo->cmdRebuild );
		if (lpErrinfo->cmdDebug)
			SetDlgItemText( hDlg, IDD_TEX_PRINT2, lpErrinfo->cmdDebug );
		if (lpErrinfo->cmdExec)
			SetDlgItemText( hDlg, IDD_TEX_PRINT3, lpErrinfo->cmdExec );
		if (lpErrinfo->cmdCompileDbg)
			SetDlgItemText( hDlg, IDD_TEX_PRINT4, lpErrinfo->cmdCompileDbg );
		if (lpErrinfo->errorFunc2)
			SetDlgItemText( hDlg, IDD_TEX_SPELL1, lpErrinfo->errorFunc2 );
		if (lpErrinfo->errorFunc3)
			SetDlgItemText( hDlg, IDD_TEX_SPELL2, lpErrinfo->errorFunc3 );
		if (lpErrinfo->errorFunc1)
			SetDlgItemText( hDlg, IDD_TEX_MAIN, lpErrinfo->errorFunc1 );
		if (lpErrinfo->errorFName)
			SetDlgItemText( hDlg, IDD_TEX_ERRFNAME, lpErrinfo->errorFName );

//		flags  = lpErrinfo->flags1;
//		flags2 = lpErrinfo->flags2;
	}

	CheckDlgButton( hDlg, IDD_TEX_TEX1_CAP   ,  !!(lpErrinfo->compileFlags & ERRCMD_CAPTURE_OUTPUT));
	CheckDlgButton( hDlg, IDD_TEX_TEX2_CAP   ,  !!(lpErrinfo->makeFlags & ERRCMD_CAPTURE_OUTPUT));
	CheckDlgButton( hDlg, IDD_TEX_PRINT1_CAP   ,  !!(lpErrinfo->rebuildFlags & ERRCMD_CAPTURE_OUTPUT));
	CheckDlgButton( hDlg, IDD_TEX_PRINT2_CAP   ,  !!(lpErrinfo->debugFlags & ERRCMD_CAPTURE_OUTPUT));
	CheckDlgButton( hDlg, IDD_TEX_PRINT3_CAP   ,  !!(lpErrinfo->execFlags & ERRCMD_CAPTURE_OUTPUT));
//	CheckDlgButton( hDlg, IDD_TEX_PRINT4_CAP   ,  !!(lpErrinfo->execFlags & ERRCMD_CAPTURE_OUTPUT));
	CheckDlgButton( hDlg, IDD_TEX_SPELL1_CAP   ,  !!(lpErrinfo->compileDFlags & ERRCMD_CAPTURE_OUTPUT));
//	CheckDlgButton( hDlg, IDD_TEX_SPELL2_CAP   ,  !!(lpErrinfo->compileDFlags & ERRCMD_CAPTURE_OUTPUT));
	CheckDlgButton( hDlg, IDD_TEX_TEX1_NC   ,  !!(lpErrinfo->compileFlags & ERRCMD_NOSHELL));
	CheckDlgButton( hDlg, IDD_TEX_TEX2_NC   ,  !!(lpErrinfo->makeFlags & ERRCMD_NOSHELL));
	CheckDlgButton( hDlg, IDD_TEX_PRINT1_NC   ,  !!(lpErrinfo->rebuildFlags & ERRCMD_NOSHELL));
	CheckDlgButton( hDlg, IDD_TEX_PRINT2_NC   ,  !!(lpErrinfo->debugFlags & ERRCMD_NOSHELL));
	CheckDlgButton( hDlg, IDD_TEX_PRINT3_NC   ,  !!(lpErrinfo->execFlags & ERRCMD_NOSHELL));
//	CheckDlgButton( hDlg, IDD_TEX_PRINT4_NC, !!(lpErrinfo->execFlags & ERRCMD_NOSHELL));
	CheckDlgButton( hDlg, IDD_TEX_SPELL1_NC   ,  !!(lpErrinfo->compileDFlags & ERRCMD_NOSHELL));
//	CheckDlgButton( hDlg, IDD_TEX_SPELL2_NC   ,  !!(lpErrinfo->compileDFlags & ERRCMD_NOSHELL));
	CheckDlgButton( hDlg, IDD_TEX_TEX1_SAF   ,  !!(lpErrinfo->compileFlags & ERRCMD_SAVEALL));
	CheckDlgButton( hDlg, IDD_TEX_TEX2_SAF   ,  !!(lpErrinfo->makeFlags & ERRCMD_SAVEALL));
	CheckDlgButton( hDlg, IDD_TEX_PRINT1_SAF   ,  !!(lpErrinfo->rebuildFlags & ERRCMD_SAVEALL));
	CheckDlgButton( hDlg, IDD_TEX_PRINT2_SAF   ,  !!(lpErrinfo->debugFlags & ERRCMD_SAVEALL));
	CheckDlgButton( hDlg, IDD_TEX_PRINT3_SAF   ,  !!(lpErrinfo->execFlags & ERRCMD_SAVEALL));
//	CheckDlgButton( hDlg, IDD_TEX_PRINT4_SAF, !!(lpErrinfo->execFlags & ERRCMD_SAVEALL));
	CheckDlgButton( hDlg, IDD_TEX_SPELL1_SAF   ,  !!(lpErrinfo->compileDFlags & ERRCMD_SAVEALL));
//	CheckDlgButton( hDlg, IDD_TEX_SPELL2_SAF   ,  !!(lpErrinfo->compileDFlags & ERRCMD_SAVEALL));
//	CheckDlgButton( hDlg, IDD_TEX_TEX1_VDOS   ,  !!(lpErrinfo->compileFlags & ERRCMD_USEVDOS));
//	CheckDlgButton( hDlg, IDD_TEX_TEX2_VDOS   ,  !!(lpErrinfo->makeFlags & ERRCMD_USEVDOS));
//	CheckDlgButton( hDlg, IDD_TEX_PRINT1_VDOS   ,  !!(lpErrinfo->rebuildFlags & ERRCMD_USEVDOS));
//	CheckDlgButton( hDlg, IDD_TEX_PRINT2_VDOS   ,  !!(lpErrinfo->debugFlags & ERRCMD_USEVDOS));
//	CheckDlgButton( hDlg, IDD_TEX_PRINT3_VDOS   ,  !!(lpErrinfo->execFlags & ERRCMD_USEVDOS));
//	CheckDlgButton( hDlg, IDD_TEX_SPELL1_VDOS   ,  !!(lpErrinfo->compileDFlags & ERRCMD_USEVDOS));

	CheckRadioButton( hDlg, IDD_TEX_TEX1_MIN, IDD_TEX_TEX1_MAX,
		(lpErrinfo->compileFlags & ERRCMD_WIN_MIN)
			? IDD_TEX_TEX1_MIN
			: ((lpErrinfo->compileFlags & ERRCMD_WIN_MAX)
				? IDD_TEX_TEX1_MAX
				: IDD_TEX_TEX1_NORMAL));
	CheckRadioButton( hDlg, IDD_TEX_TEX2_MIN, IDD_TEX_TEX2_MAX,
		(lpErrinfo->makeFlags & ERRCMD_WIN_MIN)
			? IDD_TEX_TEX2_MIN
			: ((lpErrinfo->makeFlags & ERRCMD_WIN_MAX)
				? IDD_TEX_TEX2_MAX
				: IDD_TEX_TEX2_NORMAL));
	CheckRadioButton( hDlg, IDD_TEX_PRINT1_MIN, IDD_TEX_PRINT1_MAX,
		(lpErrinfo->rebuildFlags & ERRCMD_WIN_MIN)
			? IDD_TEX_PRINT1_MIN
			: ((lpErrinfo->rebuildFlags & ERRCMD_WIN_MAX)
				? IDD_TEX_PRINT1_MAX
				: IDD_TEX_PRINT1_NORMAL));
	CheckRadioButton( hDlg, IDD_TEX_PRINT2_MIN, IDD_TEX_PRINT2_MAX,
		(lpErrinfo->debugFlags & ERRCMD_WIN_MIN)
			? IDD_TEX_PRINT2_MIN
			: ((lpErrinfo->debugFlags & ERRCMD_WIN_MAX)
				? IDD_TEX_PRINT2_MAX
				: IDD_TEX_PRINT2_NORMAL));
	CheckRadioButton( hDlg, IDD_TEX_PRINT3_MIN, IDD_TEX_PRINT3_MAX,
		(lpErrinfo->execFlags & ERRCMD_WIN_MIN)
			? IDD_TEX_PRINT3_MIN
			: ((lpErrinfo->execFlags & ERRCMD_WIN_MAX)
				? IDD_TEX_PRINT3_MAX
				: IDD_TEX_PRINT3_NORMAL));
	CheckRadioButton( hDlg, IDD_TEX_SPELL1_MIN, IDD_TEX_SPELL1_MAX,
		(lpErrinfo->compileDFlags & ERRCMD_WIN_MIN)
			? IDD_TEX_SPELL1_MIN
			: ((lpErrinfo->compileDFlags & ERRCMD_WIN_MAX)
				? IDD_TEX_SPELL1_MAX
				: IDD_TEX_SPELL1_NORMAL));
}

 /*
 ** dlgBox
 *
 * Put up a named dialog box and return the exit code.  If the dialog box
 * cannot be found or some other error occurs, return FALSE.
 *
 */
PRIVATE BOOL
dlgBox(HINSTANCE hInst, LPSTR name, HWND hWnd,
		BOOL (DLL *proc)(HWND, UINT, WPARAM, LPARAM))
{
	int stat;

	//MsgNotify("should now open TeX Dialog (dlgBox)");
	stat = DialogBox(hInst, name, hWnd, (DLGPROC)proc);
	if (stat == -1)
		stat = 0;
	return(stat);
}


/** private functions **/

/*
 * Stash the directory of the last executed command
 */
static LPSTR commandDir = NULL;

void execSetCmndDir(LPSTR newdir)
{
	if (commandDir)
	{
		MemFree(commandDir);
		commandDir = NULL;
	}
		
	if (newdir && *newdir)
		if ((commandDir = MemAlloc(strlen(newdir) + 1)) != NIL)
			strcpy(commandDir, newdir);
}

void DLL
BufWriteNonSystem(void)
{
    HBUFFER LastBuffer = WinQAttached(WinQLastCurrent());
    HBUFFER ThisBuffer = BufQCurrentBuffer();
    
    if (BufQSysFlags() & BUFFER_SYSTEM)
    {
        if (LastBuffer)
        {
            BufSetCurrentBuffer(LastBuffer);
            BufWrite();
            BufSetCurrentBuffer(ThisBuffer);
        }
    }
    else
        BufWrite();
}


PUBLIC int
execBuildCommand( LPSTR fname, LPSTR cmnd, LPErrorInfo lpErrinfo, 
		DWORD flags )
{
	UINT  wflags       = 0;    /* 0 = Normal window */
	LPSTR encodedFName = NIL;
	LPSTR shell        = NIL;
	LPSTR wrkdir       = NIL;
	LPSTR prevdir      = NIL;
	int   retcode      = -1;
	BOOL  filesSaved   = FALSE;
	char  buf[ 50 ];
	LPSTR p;
	BOOL  showMsg      = TRUE;
	LPErrorInfo prjlpErrinfo;

	if (SysQSystemFlags() & SYSTEM_DOS_COMMAND_RUNNING )
		flags &= ~ERRCMD_USEVDOS;

	if (cmnd && *cmnd)
	{
		if (flags & ERRCMD_USEVDOS)
			flags &= ~(ERRCMD_CAPTURE_OUTPUT | ERRCMD_NOSHELL);

		if (flags & ERRCMD_SAVE)
		{
			BufWriteNonSystem();
			filesSaved = TRUE;
		}
		if (flags & ERRCMD_SAVEALL)
		{
			wsprintf( buf, "fileCommand 0 IDM_FILE_SAVE_ALL" );
			LibFunctionExec( buf );
			filesSaved = TRUE;
		}
		if (filesSaved == FALSE)
		{
			if (BufQModifiedCount( FALSE ))
			{
				p = MsgStrString(ModuleInstance, MSG_TBL, IDS_SAVE_BEFORE_MAKE,
						"Save Files");
				wsprintf(buf, "DlgBufferSave 0 '%s'", p);
				StrFree(p);
				if (!LibFunctionExec(buf))
					return NOVAL;
			}
		}

		cmnd = StrNew( cmnd );

		if (lpErrinfo && (flags & ERRCMD_CAPTURE_OUTPUT))
		{
			cmnd = StrAppend( cmnd, " > " );
			cmnd = StrAppend( cmnd, 
						lpErrinfo->errorFName && *lpErrinfo->errorFName
							? lpErrinfo->errorFName
							: "%b.err" );
		}

		SetResponseFilename( NIL );

		encodedFName = cmnd;
		cmnd = DecodeFName( (fname && *fname) ? fname : BufQFilename(),
				cmnd, DECODE_LOWER | DECODE_PROJECT | DECODE_PERCENT | DECODE_MACROS);
		StrFree( encodedFName );

		if (cmnd && *cmnd)
		{
			/* If a response filename was mentioned, write out the
			 * response data to it.
			 */
			if (lpErrinfo)
				WriteResponseFileData( lpErrinfo->response );

			/* Save the current directory and switch to the directory
			 * specified in the ->wrkdir element of the prjErrorInfo
			 * structure.
			 */
			prevdir = SysQCwd( 0 );
			prjlpErrinfo = (LPErrorInfo) _ExtDescriptionEntry( PROJECT_DESC);
			if ((TexQFlags() & FLAG_TEX_MAIN) && prjlpErrinfo)
				wrkdir = prjlpErrinfo->wrkdir;
			if (wrkdir && *wrkdir)
			{
				LPSTR tmp = wrkdir = DecodeFName( BufQFilename(),
						wrkdir, DECODE_LOWER | DECODE_PROJECT);
				wrkdir = StrNormalizePath( tmp );
				StrFree( tmp );
			}
			else
				wrkdir = TransformFilename( BufQFilename(), "%v%p" );
			
			if (wrkdir && *wrkdir)
				if (wrkdir[ lstrlen( wrkdir ) - 1 ] == ':')
					wrkdir = StrAppend( wrkdir, "\\" );

			if (!wrkdir || !*wrkdir || SysSetCwd( wrkdir ))
			{
				if (wrkdir && *wrkdir)
					execSetCmndDir(wrkdir);
				if (flags & ERRCMD_WIN_MIN)
					wflags = EXEC_WINDOW_MINIMIZED;
				else if (flags & ERRCMD_WIN_MAX)
					wflags = EXEC_WINDOW_MAXIMIZED;

#ifdef WIN32
				shell = StrNew( NULL );
#else
				shell = TransformFilename( SysQArgv( 0 ), "%v%p" );
#endif
				if (flags & ERRCMD_WIN_BACKGROUND)   /* Background? */
				{
					wflags |= EXEC_WINDOW_BACKGROUND;
					shell  = StrAppend( shell, BG_SHELL );
					if (FileExists(shell) == FALSE)
					{
						StrFree( shell );
						shell = StrNew( BG_SHELL );
					}
				}
				else
				{
					wflags |= EXEC_WINDOW_EXCLUSIVE;
					shell  = StrAppend( shell, FG_SHELL );
					if (FileExists(shell) == FALSE)
					{
						StrFree( shell );
						shell = StrNew( FG_SHELL );
					}
				}

				shell = isCommandSpecified(cmnd) ? shell : StrAppend( shell, "/c " );


				if (lpErrinfo)
				{
					ErrorNextNew( -1, NIL );   // be sure to use error output, not grep
					ErrorFileDelete();
					ErrorNextNew( -1, NIL );   // be sure to use error output, not grep
				}


				if ((flags & ERRCMD_NOSHELL) != 0) 
				{
					StrFree( shell );
					shell = StrNew( " " );
				}


				if (lpErrinfo)
				{
					if (((flags & ERRCMD_USEVDOS) != 0) 
						// && ((flags & ERRCMD_CAPTURE_OUTPUT) == 0) 
						// && ((flags & ERRCMD_NOSHELL) == 0) 
						&& (LibPreLoad( SysQMapName( "cwdosbox" )))
						&& (LibFunctionExists( "_DosExecEvent" )))
					{
						SysSetSystemFlags( SYSTEM_DOS_COMMAND_STARTED 
							| ((flags & (ERRCMD_USER_TOOL|ERRCMD_NOTBUILD))
								? SYSTEM_COMMAND_RUNNING
								: SYSTEM_BUILD_RUNNING ));
						StrFree( shell );
						shell = StrNew( " " );
					}
				}

				retcode = SysExec( cmnd, shell, wflags );

				SysSetCwd( prevdir );

				if (lpErrinfo)
				{
					/* Someone cleared the DOS command flag so it must
					 * be running in 'OUR' DOS window.
					 */
					if (!(SysQSystemFlags() & SYSTEM_DOS_COMMAND_STARTED )
						&& (SysQSystemFlags() & SYSTEM_BUILD_RUNNING ))
					{
						OutputWindow( OutputQBuildID() );
						OutputSetActiveID( OutputQBuildID(), FALSE, FALSE );
						retcode = -2;
					}
					else
						SysClrSystemFlags( SYSTEM_DOS_COMMAND_STARTED 
								| SYSTEM_BUILD_RUNNING );  
				}
			}
			else if ((p = MsgStrPrintf(ModuleInstance, MSG_TBL,
					IDS_CANT_SET_CWD, NIL, "p", wrkdir)) != NIL)
			{
				MsgWarning(p);
				StrFree(p);
				showMsg = FALSE;
			}

			StrFree( cmnd );
			StrFree( shell );
			StrFree( prevdir );
			StrFree( wrkdir );
		}
	}

	if ((retcode == -1) && showMsg)
		MsgStrMessage(ModuleInstance, MSG_TBL, IDS_CMD_BUILD_ERR, MSG_ERROR);

	return retcode;
}



/*
 * Decode a command line which contains escaped characters in the form
 * %<c>.  The recognized encodings are:
 *
 *   %%   - %
 *   %v   - volume
 *   %p   - path
 *   %r   - root
 *   %d   - directory
 *   %e   - extension
 *   %b   - basename
 *   %x   - project path
 *   %y   - project root
 *
 */
LPSTR DLL
DecodeFName( LPSTR fname, LPSTR cmnd, int flags )
{
	LPSTR ptr;
	LPSTR result = NIL;
	int   len;
	LPSTR TFn = "%x";  /* dummy string to call TransformFilename() with */
	LPSTR tmp;
	char type;
	LPSTR proj = NIL;

	if ((fname != NIL) && (cmnd != NIL))
	{
		ptr = cmnd;
		len = 0;
		if (flags & DECODE_PROJECT)
			proj = ProjQCurrent();
		while (*ptr)
		{
			if (*ptr == '%')
			{
				switch( ptr[ 1 ] )
				{
				case 'x':
				case 'X':
					type = (char)((proj != NIL) ? 'x' : '\0');
					break;
				case 'y':
				case 'Y':
					type = (char)((proj != NIL) ? 'y' : '\0');
					break;

				case 'v':
				case 'p':
				case 'r':
				case 'd':
				case 'e':
				case 'b':
					type = (char)((flags & DECODE_LOWER) ? ptr[1] : '\0');
					break;
				case 'V':
				case 'P':
				case 'R':
				case 'D':
				case 'E':
				case 'B':
					type = (char)((flags & DECODE_UPPER) ?
							tolower((BYTE)ptr[1]) : '\0');
					break;
				case '%':
					type = (char)((flags & DECODE_PERCENT) ? '%' : '\0');
					break;
				default:
					type = '\0';
					break;
				}
				if (type == '\0')
				{
					/* unrecognized escape sequence, pass it untouched */
					len += 2;
					ptr += 2;
				}
				else
				{
					if (len)
						result = StrNAppend( result, cmnd, len );
					TFn[1] = type;

					if (type == 'x')
						tmp = TransformFilename( ProjQCurrent(), "%v%p" );
					else if (type == 'y')
						tmp = TransformFilename( ProjQCurrent(), "%r" );
					else
						tmp = TransformFilename( fname, TFn );

					result = StrAppend( result, tmp );
					StrFree( tmp );

					len  = 0;
					ptr += 2;
					cmnd = ptr;
				}
			}
			else
			{
				len++;
				ptr++;
			}
		}

		if (len)
			result = StrNAppend( result, cmnd, len );
	}

	return result;
}

static BOOL
isCommandSpecified( LPSTR fname )
{
	BOOL stat = FALSE;

	if (fname)
	{
		while (*fname)
		{
			if (*fname == '/' || *fname == '-')
			{
				fname++;
				if (*fname == 'c' || *fname == 'C')
				{
					stat = TRUE;
					break;
				}
				while (*fname && !isspace( (BYTE)*fname ))
					fname++;
			}
			else if (isalpha( (BYTE)*fname ))
				break;
			fname++;
		}
	}
	return stat;
}



PRIVATE void 
ascii_to_ansi ()
{
	MarkID cmark;
	cmark = MarkAllocLocal();
	MarkSetColumnAdjust( cmark, FALSE, TRUE );

	MsgMessage("Translating accents from ASCII to ANSI, wait ...");
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
	MarkGoto( cmark );
	MarkFree( cmark );
	MsgMessage ("Translation from ASCII to ANSI completed");
}

PRIVATE void 
ansi_to_ascii ()
{
	MarkID cmark;
	cmark = MarkAllocLocal();
	MarkSetColumnAdjust( cmark, FALSE, TRUE );

	MsgMessage("Translating accents from ANSI to ASCII, wait ...");
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
	MarkGoto( cmark );
	MarkFree( cmark );
	MsgMessage ("Translation from ANSI to ASCII completed");
}

PRIVATE void 
ascii_to_tex ()
{
	MarkID cmark;
	cmark = MarkAllocLocal();
	MarkSetColumnAdjust( cmark, FALSE, TRUE );

	MsgMessage("Translating accents from ASCII to TeX, wait ...");
		SrchTranslate ("", "{\\\\pounds}", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\^{a}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\\"{a}",    SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\`{a}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\^{e}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\\"{e}",    SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\`{e}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\'{e}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\^{\\\\i}",   SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\\"{\\\\i}",  SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\^{o}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\\"{o}",    SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\^{u}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\\"{u}",    SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\`{u}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\c{c}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
	MarkGoto( cmark );
	MarkFree( cmark );
	MsgMessage ("Translation from ASCII to TeX completed");
}

PRIVATE void 
ansi_to_tex ()
{
	MarkID cmark;
	cmark = MarkAllocLocal();
	MarkSetColumnAdjust( cmark, FALSE, TRUE );

	MsgMessage("Translating accents from ANSI to TeX, wait ...");
		SrchTranslate ("", "{\\\\pounds}", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\^{a}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\\"{a}",    SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\`{a}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\^{e}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\\"{e}",    SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\`{e}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\'{e}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\^{\\\\i}",   SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\\"{\\\\i}",  SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\^{o}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\\"{o}",    SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\^{u}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\\"{u}",    SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\`{u}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("", "\\\\c{c}",     SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
	MarkGoto( cmark );
	MarkFree( cmark );
	MsgMessage ("Translation from ANSI to TeX completed");
}

PRIVATE void 
tex_to_ascii ()
{
	MarkID cmark;
	cmark = MarkAllocLocal();
	MarkSetColumnAdjust( cmark, FALSE, TRUE );

	MsgMessage("Translating accents from TeX to ASCII, wait ...");
		SrchTranslate ("{\\\\pounds}", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^{a}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"{a}",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\`{a}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^{e}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"{e}",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\`{e}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\'{e}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^{\\\\i}",   "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"{\\\\i}",  "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^{o}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"{o}",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^{u}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"{u}",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\`{u}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\c{c}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\pounds", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^a",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"a",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\`a",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^e",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"e",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\`e",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\'e",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^\\\\i",   "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"\\\\i",  "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^o",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"o",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^u",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"u",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\`u",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
	MarkGoto( cmark );
	MarkFree( cmark );
	MsgMessage ("Translation from TeX to ASCII completed");
}

PRIVATE void 
tex_to_ansi ()
{
	MarkID cmark;
	cmark = MarkAllocLocal();
	MarkSetColumnAdjust( cmark, FALSE, TRUE );

	MsgMessage("Translating accents from TeX to ANSI, wait ...");
		SrchTranslate ("{\\\\pounds}", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^{a}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"{a}",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\`{a}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^{e}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"{e}",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\`{e}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\'{e}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^{\\\\i}",   "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"{\\\\i}",  "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^{o}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"{o}",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^{u}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"{u}",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\`{u}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\c{c}",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\pounds", "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^a",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"a",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\`a",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^e",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"e",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\`e",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\'e",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^\\\\i",   "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"\\\\i",  "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^o",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"o",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\\^u",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\\"u",    "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
		SrchTranslate ("\\\\`u",     "", SEARCH_REGEX | SEARCH_WRAP | SEARCH_GLOBAL, NIL);
	MarkGoto( cmark );
	MarkFree( cmark );
	MsgMessage ("Translation from TeX to ANSI completed");
}


/*
 ** TabToTeXTabular()
 *
 *  FILENAME: D:\CW32\TEX\TEXMENU.C
 *
 *  PARAMETERS:	 NONE
 *
 *  DESCRIPTION: Typically converts tab delimited data
 *				 into a TeX table (tabular) 
 *
 *  RETURNS:
 *
 */

PUBLIC void DLL 
TabToTeXTabular(  )
{
	int on, oncw;
	long endline, startline, numb_endline;
 	MarkID selmark, endmark, xmark;

	MarkNormalizeSel();
	MarkAdjustSel();
	selmark = MarkQSelBeginID();
	endmark = MarkQSelEndID();
	//MsgNotify(StrItoA(selmark,10));
	if ( selmark == MARK_INVALID )
	{
		MsgNotify("Nothing selected !");
		return;
	}

 	xmark = MarkAllocLocal();
 	MarkSetColumnAdjust( xmark, FALSE, TRUE );
 //	MarkSetColumnAdjust( MARK_SELECTION, TRUE, FALSE );

	on = TexWrapEnable(-1);
	TexWrapEnable(0);
	oncw = WrapEnable(-1);
	WrapEnable(0);
	MsgMessage("Converting Tab delimited data into tabular form, wait ...");

	//endmark = MarkQSelEndID();
	
	MarkGoto(endmark);
	endline = BufQCurrentLine() + 1;
	MarkGoto(selmark);
	startline = BufQCurrentLine();
	numb_endline = endline - startline;

	while ( numb_endline > 0 )
	{
		SrchTranslate("\\n", "\\\\\\\\@_@", SEARCH_REGEX |SEARCH_ONCE| SEARCH_SELECTION, NIL);
		numb_endline--;
		//MsgNotify(StrItoA(numb_endline,10));
	}

	MarkNormalizeSel();
	MarkAdjustSel();

	//	MsgNotify("first change");
	SrchTranslate("\\t", "\\&", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("( *)\\&( *)", "\\&", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION | SEARCH_MAXIMAL, NIL);
	//	MsgNotify("second change");
	SrchTranslate("\\&", "\\t\\& ", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);

	//selmark = MarkQSelBeginID();

	//endmark = MarkQSelEndID();
	//MarkGoto(endmark);
	//endline = BufQCurrentLine() + 1;
		
	MarkGoto(selmark);
	MovHome();
  	while (endline > BufQCurrentLine())
  	{
   		if (SrchTranslate("@_@", "", SEARCH_REGEX | SEARCH_ONCE | SEARCH_FORWARD, NIL))
  		{
  			if ( endline > BufQCurrentLine() )
  			{
  				 BufInsertEOL();
				 //MovHome();
  				 //endline++;
  			}
  			else 
  				break;
  		}
  		else
  			break;
  	}
	
	MarkGoto(endmark);

	SrchTranslate("^( )*", " ", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION|SEARCH_MAXIMAL, NIL);
	WrapEnable(oncw);
	TexWrapEnable(on);

 	MarkGoto( xmark );
 	MarkFree( xmark );
	MsgMessage("Conversion of tab delimited data completed !");

	return;
}

/*
 ** TeXTabularToTab()
 *
 *  FILENAME: D:\CW32\TEX\TEXMENU.C
 *
 *  PARAMETERS:	 NONE
 *
 *  DESCRIPTION: Typically converts a TeX table (tabular)
 *				 into tab delimited data 
 *
 *  RETURNS:
 *
 */

PUBLIC void DLL 
TeXTabularToTab(  )
{
	
	int on, oncw;
	long endline, startline, numb_endline;
 	MarkID selmark, endmark, xmark;

	MarkAdjustSel();
	MarkNormalizeSel();
	selmark = MarkQSelBeginID();
	endmark = MarkQSelEndID();
	//MsgNotify(StrItoA(selmark,10));
	if ( selmark == MARK_INVALID )
	{
		MsgNotify("Nothing selected !");
		return;
	}
   	xmark = MarkAllocLocal();
	MarkSetColumnAdjust( xmark, FALSE, TRUE );
 	//MarkSetColumnAdjust( MARK_SELECTION, TRUE, FALSE );
	MsgMessage("Converting TeX table into Tab delimited data, wait ...");

	on = TexWrapEnable(-1);
	TexWrapEnable(0);
	oncw = WrapEnable(-1);
	WrapEnable(0);

	MarkGoto(endmark);
	endline = BufQCurrentLine() + 1;
	MarkGoto(selmark);
	startline = BufQCurrentLine();
	numb_endline = endline - startline;

	MarkGoto(selmark);
	while ( numb_endline - 1 > 0 )
	{
		SrchTranslate("\\n", " ", SEARCH_REGEX |SEARCH_ONCE| SEARCH_SELECTION, NIL);
		numb_endline--;
		//MsgNotify(StrItoA(numb_endline,10));
	}
	MarkAdjustSel();
	MarkNormalizeSel();
	selmark = MarkQSelBeginID();
	endmark = MarkQSelEndID();
	MarkGoto(endmark);
	endline = BufQCurrentLine() + 1;
	MarkGoto(selmark);
	startline = BufQCurrentLine();
	numb_endline = endline - startline;
	MarkGoto(endmark);

	SrchTranslate("(\\\\\\\\)", "x@xx@x",  SEARCH_WRAP |SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION|SEARCH_MAXIMAL, NIL);
	SrchTranslate("([ \\t])+\\&([ \\t])+", "\\&", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION|SEARCH_MAXIMAL, NIL);
	SrchTranslate("([\\t ])+", "@_@", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION|SEARCH_MAXIMAL, NIL);
	SrchTranslate("\\&", "\\t", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);

	MarkNormalizeSel();
	selmark = MarkQSelBeginID();
	endmark = MarkQSelEndID();
	MarkGoto(selmark);
	startline = BufQCurrentLine();
	numb_endline = endline - startline;
	MarkGoto(endmark);
	endline = BufQCurrentLine() + 1;

	numb_endline = 0;
	while (startline > 0)
	{
 		if (SrchFind("x@x\\cx@x", SEARCH_REGEX | SEARCH_ONCE , NIL))
		{
			if ( startline >= BufQCurrentLine() )
			{
				 BufInsertEOL();
				 numb_endline++;
				 continue;
			}
		}
		break;
	}

	//MarkAdjustSel();
	//MarkNormalizeSel();
	selmark = MarkQSelBeginID();
	endmark = MarkQSelEndID();

	MarkGoto(endmark);
	MovRight(1);
	SrchTranslate("x@x", "", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION|SEARCH_MAXIMAL, NIL);
	SrchTranslate(" +", "", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION|SEARCH_MAXIMAL, NIL);
	SrchTranslate("^(@_@)+", "", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION|SEARCH_MAXIMAL, NIL);
	SrchTranslate("(@_@)+", " ", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION|SEARCH_MAXIMAL, NIL);
	MovLeft(1);
	SrchTranslate("(@_@)+", " ",  SEARCH_REGEX |SEARCH_ONCE | SEARCH_FORWARD | SEARCH_MAXIMAL, NIL);

	WrapEnable(oncw);
	TexWrapEnable(on);

 	MarkGoto( xmark );
 	MarkFree( xmark );
 	MarkFree( selmark );
 	MarkFree( endmark );
 	MsgMessage("Conversion into tab delimited data completed !");
	
	return;
}


/*
 ** TextToTeX()
 *
 *  FILENAME: D:\CW32\TEX\TEXMENU.C
 *
 *  PARAMETERS:	 linebreak = 0 if no line breaks
 *						   = 1 if line breaks
 *
 *  DESCRIPTION: Makes basic reformating of a word file
 *               exported as a text only file 
 *
 *  RETURNS:
 *
 */

PUBLIC void DLL 
TextToTeX( int linebreak )
{
	
	int on, oncw;
	MarkID xmark;
	xmark = MarkAllocLocal();
	MarkSetColumnAdjust( xmark, FALSE, TRUE );
	
	on = TexWrapEnable(-1);
	TexWrapEnable(0);
	oncw = WrapEnable(-1);
	WrapEnable(0);
	MsgMessage("Converting Text to TeX special signs, wait ...");

	if (linebreak == 0)
	{
		SrchTranslate("\\n", "\\n\\n ", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION , NIL);
		SrchTranslate("\\n[ \\t]*\\n\\n", "\\n\\n", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
		SrchTranslate("^  ", " ", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL, NIL);
	}
   	SrchTranslate("^\\\\)", " \\\\", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
   	SrchTranslate("\\n[ \\t]*\\n([^\\t\\n])", "\\n\\n \\1", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);

	SrchTranslate("\\\\", "\\\\backslash ", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\&", "\\\\\\&", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("%", "\\\\%", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("_", "\\\\_", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("#", "\\\\#", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	//SrchTranslate("\\[", "\\\\\\[", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	//SrchTranslate("\\]", "\\\\\\]", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\{", "\\\\{", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\}", "\\\\}", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\$", "\\\\\\$", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("", "\\\\pounds ", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("", "\'", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("", "<<", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("", ">>", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("", "\\{\\\\oe\\}", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	//SrchTranslate("", "\\\\raisebox{\\.6ex}{\\$\\\\circ\\$}", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\^", "\\$\\^\\\\wedge\\$", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("", "\\$\\^\\\\circ\\$", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("~", "\\$\\^\\\\sim\\$", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("", "\\{\\\\S\\}", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("([ \\t\\n\\(])\"([^ \\t\\n])", "\\1``\\2", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("([^ \\t\\n])\"([ \\t\\n\\)])", "\\1\'\'\\2", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("([ \\t\\n])\'([^ \\t\\n])", "\\1`\\2", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);

	TexWrapAll();

	WrapEnable(oncw);
	TexWrapEnable(on);

	MarkGoto( xmark );
	MarkFree( xmark );
	MsgMessage("Text to TeX convertion completed");
}

/*
 ** TeXToText()
 *
 *  FILENAME: D:\CW32\TEX\TEXMENU.C
 *
 *  PARAMETERS:	 linebreak = 0 if no line breaks
 *						   = 1 if line breaks
 *
 *  DESCRIPTION: Makes basic reformating of a word file
 *               exported as a text only file 
 *
 *  RETURNS:
 *
 */

PUBLIC void DLL 
TeXToText( int linebreak )
{
	int on, oncw;
	MarkID xmark;
	xmark = MarkAllocLocal();
	MarkSetColumnAdjust( xmark, FALSE, TRUE );
	
	on = TexWrapEnable(-1);
	TexWrapEnable(0);
	oncw = WrapEnable(-1);
	WrapEnable(0);

	MsgMessage("Converting TeX special signs to text, wait ...");
	if (linebreak == 0)
	{
		SrchTranslate("([ \\t]*)\\n([^ \\t\\n])", " \\2", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION , NIL);
//		SrchTranslate("\\n", "\\n\\n ", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION , NIL);
//		SrchTranslate("\\n[ \\t]*\\n\\n", "\\n\\n", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
//		SrchTranslate("^  ", " ", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL, NIL);
	}
	SrchTranslate("\\$\\^\\\\circ\\$", "", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\$\\^\\\\sim\\$", "~", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\$\\^\\\\wedge\\$", "\\^", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\\\\\&","\\&", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\\\%","%", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("{\\\\S}", "", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\\\S([^a-zA-Z@])", "\\1", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("{\\\\oe}", "", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\\\pounds([^A-Za-z@])", "\\1", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\\\\\$", "\\$", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\\\\\{", "\\{", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\\\\\}", "\\}", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\\\_", "_", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\\\#", "#", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("``", "\"", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\'\'", "\"", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("([^ \\t\\n])'([ \\t\\n,\\.:;!\\?])", "\\1\\2", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("([ \\t\\n])`([^ \\t\\n,\\.:;!\\?'])", "\\1\'\\2", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\\\backslash[ \\t]|\\{\\\\backslash\\}", "\\\\", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("\\\\backslash([^a-zA-Z@])", "\\\\\\1", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate("<<", "", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);
	SrchTranslate(">>", "", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION, NIL);

	SrchTranslate("^ ", "", SEARCH_WRAP | SEARCH_REGEX |SEARCH_GLOBAL| SEARCH_SELECTION , NIL);

	WrapEnable(oncw);
	TexWrapEnable(on);

	MarkGoto( xmark );
	MarkFree( xmark );
	MsgMessage("TeX to text convertion completed");
}



