/*******************************************************************
 *
 *    DESCRIPTION: Modified Word Wrap 
 *				   Part of TeX support for CodeWRight (TEX.DLL).
 *
 *    AUTHOR: 	Gilbert Ritschard, Department of Econometrics
 *				University of Geneva, CH-1211 Geneva 4
 *				e-mail: ritschar@uni2a.unige.ch
 *
 *    HISTORY:  6-sept, 1994 created
 *				November 1, 1996 (adapted for CW 4)
 *
 *		Part of the code is taken from "wwrap.c" (C)CodeWRight
 *
 *******************************************************************/


//#include "win_filt.h"
#include <windows.h>
#include "exports.h"
#include "cwstart.h"
#include "tex.h"

#define WRAP_DISABLED              0    /* line wrapping is disabled   */
#define WRAP_ENABLED               1    /* line wrapping is enabled    */
#define WRAP_CONTINUOUS            2    /* line wrapping it continuous */


PRIVATE int  _wrap_line( BOOL join );
PRIVATE void adjustCurrentLine( void );
//PRIVATE void insertIndentedEOL( void );
PRIVATE void insertNotIndentedEOL( void );
PRIVATE void setRightMargin( void );
PRIVATE void trimWhite(long maxWhite);
PRIVATE int  XNEAR getWWMode( void );
PRIVATE int  XNEAR getTexWWMode( void );

PRIVATE long XNEAR wRightMargin = 0;
PRIVATE long XNEAR scrWidth = 0;
//PRIVATE int  XNEAR ww_enabled = 0;
PRIVATE	int	wrap_event_mode = 0;
PRIVATE int tex_wrap_mode = 0;



/*
 ** TeXWrapEnable()
 *
 *  PARAMETERS:
 *    on  - 0 : WRAP_DISABLED, line wrapping is disabled
 *          1 : WRAP_ENABLED, line wrapping is enabled
 *          2 : WRAP_CONTINUOUS, line wrapping is continuous until no more lines
 *                  will wrap.
 *
 *  RETURNS:
 *    0,1,2  : current state of line wrapping
 */
PUBLIC BOOL DLL
TexWrapEnable( int on )
{
	DWORD flags = 0L;
	DWORD flags2= 0L;

	if ( on == 0)
	{
		flags =  (TexQFlags() & ~FLAG_TEX_WRAP);
		if ( _currentBuf_wrap )
			BufSetSysFlags( BufQSysFlags() | BUFFER_WORDWRAP );
		MsgMessage( "TeX word wrap is off" );
	}
	else if ( on > 0 )
	{
		if ( BufQSysFlags() & BUFFER_WORDWRAP )
		{
			BufSetSysFlags( BufQSysFlags() & ~BUFFER_WORDWRAP );
			_currentBuf_wrap = TRUE;
		}
		flags =  (TexQFlags() | FLAG_TEX_WRAP) ;
		MsgMessage( "TeX word wrap is on" );
	}
	if ( on >= 0)
	{
 		flags2 = TexSetFlags( flags );
 //		if(flags != flags2);
 //		{
 //					MsgNotify(" TexSetFlags error ");
 //					MsgNotify( StrItoA(flags, 2) );
 //					MsgNotify( StrItoA(flags2,2));
 //					MsgNotify( StrItoA(TexQFlags(),2));
 //		}
		CheckMenuItem( MenuQHandle() , IDM_TEX_WORD_WRAP,
				( TexQFlags() & FLAG_TEX_WRAP ? MF_CHECKED : MF_UNCHECKED));
	}
	return getTexWWMode();
}

/*
 ** TexWrapSetRightMargin()
 *
 *  PARAMETERS
 *    rmargin   - specifies the right margin for word wrapping purposes.
 *
 *  DESCRIPTION
 *    The value set with this function defines the right margin used by 
 *    the word wrapping functions.  If rmargin equals zero, then the 
 *    value set for WinVisibleMarginColumn() is used as the right 
 *    margin.  If WinVisibleMarginColumn() is zero, then the width of 
 *    the window is used.
 *
 *  RETURNS
 *    The current right margin setting.
 */
//  PUBLIC long DLL
//  TexWrapSetRightMargin( long rmargin )
//  {
//  	if (ArgCount() && (rmargin >= 0))
//  		wRightMargin = rmargin;
//  	return wRightMargin;
//  }

/*
 ** ToggleTexWrapEnable
 *
 *  FILENAME: D:\CW32\TEX\TEXWWRAP.C
 *
 *  PARAMETERS:
 *
 *  DESCRIPTION:
 *
 *  RETURNS:
 *
 */

PUBLIC void DLL
ToggleTexWrapEnable(void)
{
	if ( TexWrapEnable(-1) > 0 )
	{
		TexWrapEnable(0);
	}
	else  // TeX wrap is off
	{
		TexWrapEnable(2);
	}
}
	


/*
 ** TexWrapParagraph()
 *
 *  DESCRIPTION
 *    Wrap the lines in the current paragraph.  The right margin
 *    is determined by WrapSetRightMargin().
 *
 *    A paragraph is defined as text surrounded by blanks in column 1.
 */
PUBLIC void DLL
TexWrapParagraph( void )
{
	BOOL   pastEOF = (BufReadChar() == (UINT)(-1));
	MarkID cmark;
	BOOL   moved;
	long firstch; 
	int		wrap_mode;

	if (BufQSysFlags() & BUFFER_READ_ONLY)
	{
		MsgMessage( "Buffer is read-only." );
		SysBeep();
		return;
	}


	if (IS_HEX_VIEW(BufQSysFlags()))
	{
		MsgMessage( "Cannot wrap while in Hexadecimal mode." );
		SysBeep();
		return;
	}

	wrap_mode = getWWMode();
	WrapEnable(WRAP_DISABLED);

	moved = MovPrevChar( 1L );
	cmark = MarkAllocLocal();
	if (moved)
		MovNextChar( 1L );

	setRightMargin();

	MarkSetColumnAdjust( cmark, FALSE, TRUE );

	/* if (!SrchFind( "^[ \t]*\\n\\c", SEARCH_REGEX, NIL ))	*/
	 if (!SrchFind( "^[ \\t\\n]+\\c", SEARCH_REGEX, NIL ))
		MovTopBuf();

	while (TRUE)
	{
		trimWhite(BufQLineLength());
		switch (_wrap_line( TRUE ))
		{
		case 1:  // line was formatted
			if (MovDown( 1 ))
			{
				adjustCurrentLine();
				continue;
			}
			break;
		case 2:  // line was formatted and cursor is on next line
		case 3:  // line was joined with next line
			continue;
		case 0:  // line was not formatted
			/* Check for more characters on the following line */
			/* Stop if the following line starts with a blank gr*/
			if (MovDown( 1 ))
			{
				MovHome();
			   _PosInit( -1 );
			   firstch = _PosCurrentCharNL();
			   if ( ( firstch == ' ' || firstch == '\t' ) )
			   {
			  		break;
			   } 
			 
			 	if (SrchFind( "[^ \t]|$", SEARCH_FORWARD | SEARCH_REGEX, NIL ))
				{
					_PosInit( -1 );
					if (!EOL(_PosCurrentCharNL()))
					{
						adjustCurrentLine();
						continue;
					}
				}
			}
		default:
			break;
		}
		break;
	}

	if (pastEOF)
		MovEOF(), MovEOL();
	else
	{
		/* restore the cursor position */
		MarkGoto( cmark );
		if (BufQPastEOF())
		{
			MovEOF();
			MovEOL();
		}
		else if (BufQPastEOL())
		{
			MovHome();
			MovEOL();
		}
		else if (moved)
			MovNextChar( 1L );
	}
	MarkFree( cmark );
	WrapEnable(wrap_mode);
}

//long DLL 
//_ext_ww_space_char( long eventnum, long chptr )
//{
//	return _tex_ww_space_char( eventnum, chptr );
//}
//
//long DLL 
//_ext_ww_inserted_char( long eventnum, long chptr )
//{
//	return _tex_ww_inserted_char( eventnum, chptr );
//}

//long DLL
//_tex_ww_space_char( long eventnum, long chptr )
//{
//	WrapEnable(wrap_event_mode);
////	if (getWWMode() == WRAP_ENABLED) 
////		WrapEnable(WRAP_CONTINUOUS);
//	return 1L;
//}


long DLL
_tex_ww_handler( long eventnum, long chptr )
{
	MarkID cmark;
	MarkID cmarkprev;							 /*gr 11/11/93*/
	long   prevlinenumber = BufQCurrentLine();	 /*gr 11/11/93*/
	UINT    insertedch;							 /*gr 11/11/93*/
	UINT    ch;							 /*gr 11/11/93*/
	DWORD   offset;								 /*gr 11/11/93*/
	BOOL   pastEOF;
	//BOOL   moved;
	//char   c;
	//BOOL   inTrailingWhite;
	//long   curLine;
	long   lineLength;


	if ( BufQSysFlags() & BUFFER_WORDWRAP && (getTexWWMode() != WRAP_DISABLED))
	{
		BufSetSysFlags( BufQSysFlags() & ~BUFFER_WORDWRAP );
		_currentBuf_wrap = TRUE;
	}
	
 //	if ((getTexWWMode() == WRAP_DISABLED) &&  _currentBuf_wrap )
 //	{
 //		BufSetSysFlags( BufQSysFlags() | BUFFER_WORDWRAP );
 // 	}

	if ( (getTexWWMode() == WRAP_DISABLED) || IS_HEX_VIEW(BufQSysFlags()) 
		  || !_currentBuf_wrap )
		return 0;

	if (BufQSysFlags() & BUFFER_READ_ONLY)
	{
		MsgMessage( "Buffer is read-only." );
		SysBeep();
		return 1L;
	}


	pastEOF = (BufReadChar() == (UINT)(-1));
	lineLength = BufQLineLength();
	setRightMargin();
	if (pastEOF && BufQCurrentCol() < scrWidth)
		return 1L;

	cmark = MarkAllocLocal();
	MarkSetColumnAdjust( cmark, FALSE, TRUE );

	/*    Added  for fixing a bug  19.2.93
	 *  We set a bookmark at the position before the inserted character
	 *  After the wrapping, the current position will be restored 
	 *  just one position after this bookmark. This is to get a more robust
	 *  saved position.
	 */
	MovLeft(1L);								  /*gr 11/11/93*/
		insertedch = BufReadChar();				  /*gr 11/11/93*/
	cmarkprev = MarkAllocLocal();				  /*gr 11/11/93*/
	MovRight(1L);								  /*gr 11/11/93*/
	MarkSetColumnAdjust( cmarkprev, FALSE, TRUE );/*gr 11/11/93*/
	/*    see test at exit  */

	/* trim trailing whitespace */
	trimWhite(lineLength);

	do
	{
		switch (_wrap_line( getTexWWMode() == WRAP_CONTINUOUS ))
		{
		case 1:  // line was formatted
			if (MovDown( 1 ))
				continue;
			break;
		case 2:  // line was formatted and cursor is on next line
		case 3:  // line was joined with next line
			continue;
		case 0:  // line was not formatted

		default:
			break;
		}
		break;
	} while (getTexWWMode() == WRAP_CONTINUOUS);

	if (pastEOF)
		MovEOF(), MovEOL();

	else
	{
		MarkGoto( cmark );
		offset = BufQOffset();

		/* check if current pos has changed line */
		if ( prevlinenumber < BufQCurrentLine() )
		{
			adjustCurrentLine();
			MarkGoto( cmarkprev );
 			_PosInit(-1);
 			ch = _PosCurrentCharNL();
			/* if the wrap occured at a white space, 
			 * leave cursor at the cmarkprev pos */
    		if ( !EOL(ch) && ( insertedch != ch ) )
   			{
   				goto EWIL_exit;
   			}
		}
		MarkGoto( cmarkprev );
		if ( offset > BufQOffset() )
				MovRight( 1L );
		
	}

  EWIL_exit:	 
	MarkFree( cmarkprev ); /*gr 11/11/93*/
	MarkFree( cmark );
	return 1L;
	
}


/*
 * _wrap_line
 *
 * RETURNS:
 *  0 = line was not formatted
 *  1 = line was formatted
 *  2 = line was formatted and cursor is on next line
 *  3 = line was joined with next line
 */
PRIVATE int
_wrap_line( BOOL join )
{
	MarkID cmark;
	long   startline = BufQCurrentLine();
	long   lastoff = 0;
	long   endcol;
	int    status = 0;
	long   offdiff;
	int    ch;
	int    prevch;
	DWORD  ovrtype = (BufQSysFlags() & BUFFER_OVERTYPE_MODE);
	BOOL   pastEOF = (BufReadChar() == (UINT)(-1));

	cmark = MarkAllocLocal();
	MarkSetColumnAdjust( cmark, FALSE, TRUE );
	MovEOL();
	endcol = BufQCurrentCol();

	BufSetSysFlags( BufQSysFlags() & ~BUFFER_OVERTYPE_MODE );

	if (BufQCurrentCol() >= scrWidth)
	{
		while (BufQCurrentCol() > scrWidth)
		{
			_PosInit( -1 );
			ch     = _PosCurrentCharNL();
			prevch = _PosPrevCharNL();
			while (TRUE)
			{
				/* Provide a litle faster way of doing:
				 *  if (SrchFind( "^|[^ \t][ \t]", SEARCH_REGEX, NIL ) == FALSE)
				 *		goto WL_exit;
				 */

				if (EOL(prevch) 
						|| ((prevch != ' ') && (prevch != '\t')) 
							&& ((ch == ' ') || (ch == '\t')))
						break;
	
				ch = prevch;
				prevch = _PosPrevCharNL();
			}

			if (EOL(prevch))
			{
				if (lastoff)
				{
					MovToOffset( lastoff );
					break;
				}
				goto WL_exit;
			}
			lastoff = _PosQOffset();
			MovToOffset( lastoff );
		}

		if (lastoff || (BufQCurrentCol() < scrWidth))
		{
			MovNextChar( 1L );   /* MovRight( 1L );*/

			/* If we're already at the end of the current line, just restore
			 * the original position and return.
			 */
			ch = (int)BufReadChar();
			if ((ch == -1) || (ch == '\n'))
				goto WL_exit;						

			/* Split the line and turn off autoindent.
			 */
			/* insertIndentedEOL();  */
			insertNotIndentedEOL(); 

			/* Skip leading whitespace at beginning of split line. 
			 */
			_PosInit( -1 );
			ch = _PosCurrentCharNL();
			offdiff = 0;
			while ((ch == ' ') || (ch == '\t'))
			{
				offdiff++;
				ch = _PosNextCharNL();
			}
			BufDelChar( offdiff );

			if (pastEOF)
				MovEOF(), MovEOL();
			else
				MarkGoto( cmark );
			status = (BufQCurrentLine() == startline) ? 1 : 2;
		}
	}
	else if (join)
	{
		/* line is shorter than the right margin so try to join with
		 * next line.
		 */
	     _PosInit( -1 );
	     _PosCurrentCharNL();
		 ch =_PosNextCharNL();

		 if ( ch == ' ' || ch == '\t' ) goto WL_exit; 
		 
		 while (!EOL(ch))
		 {
		 	if ((ch != ' ') && (ch != '\t'))
		 	{
		 		offdiff = 0;
		 		while (!EOL(ch) && (ch != ' ') && (ch != '\t'))
		 		{
		 		offdiff++;
#ifdef DBC_ABLE
					if (IsDBChar(ch))
						offdiff++;
#endif // DBC_ABLE
		 			ch = _PosNextCharNL();
		 		}
		 		if (offdiff < scrWidth - endcol)
		 		{
		 			BufDelChar( 1 );
		 			_PosInit( -1 );
		 			_PosCurrentCharNL();
		 			ch = _PosCurrentCharNL();
		 			offdiff = 0;
		 
		 			while ((ch == ' ') || (ch == '\t'))
		 			{
		 				offdiff++;
		 				ch = _PosNextCharNL();
		 			}
		 			BufDelChar( offdiff );
		 			BufInsertStr( " " );
		 			status = 3;
		 			break;
		 		}
		 		else
		 			goto WL_exit;
		 	}
		 	ch = _PosNextCharNL();
		 }
		
	}

 WL_exit:
 	BufSetSysFlags( BufQSysFlags() | ovrtype );
 
 	if (pastEOF)
 		MovEOF(), MovEOL();
 	else
		MarkGoto( cmark );
	MarkFree( cmark );
	return status;
}


/*
 ** adjustCurrentLine
 *
 * Redo the current line's leading whitespace if necessary.
 */
PRIVATE void
adjustCurrentLine( void )
{
	WORD ch;
	long cnt = 0;

	/* create a new line with leading whitespace above the current line */
	MovUp( 1 );
	MovEOL();
	/* insertIndentedEOL(); */ 
	insertNotIndentedEOL();

	BufDelChar( 1 );  /* join with next line */

	/* count leading whitespace chars and move pos past them */
	_PosInit( -1 );
	ch = _PosCurrentCharNL();
	while ((ch == ' ') || (ch == '\t'))
	{
		ch = _PosNextChar();
		cnt++;
	}

	/* delete whitespace */
	if (cnt)
		BufDelChar( cnt );
}


//PRIVATE void
//insertIndentedEOL( void )
//{
//	DWORD  bflags;
//
//	bflags = BufQSysFlags();
//	BufSetSysFlags( bflags | BUFFER_AUTO_INDENT);
//	BufInsertEOL();
//	if (!(bflags & BUFFER_AUTO_INDENT))
//		BufSetSysFlags( BufQSysFlags() & ~BUFFER_AUTO_INDENT);
//}


PRIVATE void
insertNotIndentedEOL( void )
{
	DWORD  bflags;

	bflags = BufQSysFlags();
	BufSetSysFlags( bflags & ~BUFFER_AUTO_INDENT);
	BufInsertEOL();
	if ( (bflags & BUFFER_AUTO_INDENT) )
		BufSetSysFlags( BufQSysFlags() | BUFFER_AUTO_INDENT);
}


PRIVATE void
setRightMargin( void )
{
	if (wRightMargin)
		scrWidth = wRightMargin;
	else if (WinVisibleMarginColumn( -1 ))
		scrWidth = WinVisibleMarginColumn( -1 );
	else
		scrWidth = WinQTextWidth();
	if (scrWidth <= 0)
		scrWidth = 78;
}


PRIVATE void
trimWhite(long maxWhite)
{
	long white;
	int    ch;

	if (maxWhite <= 0)
		return;

	/* trim trailing whitespace from the current line before wrapping */
	MarkSavePos();
	MovEOL();
	_PosInit( -1 );
	for (white = 0; ((ch = _PosPrevCharNL()) == ' ') || (ch == '\t'); )
	{
		if (++white >= maxWhite)
		{
			white = maxWhite;
			break;
		}
	}
	if (white != 0)
	{
		/* whitespace present, delete it */
		MarkBeginSel(SELECTION_EXCLUSIVE);
		MovLeft(white);
		BufDelSelection();
	}
	MarkRestorePos();		
	if (BufQPastEOL())
		MovEOL();
}


PRIVATE int XNEAR
getWWMode( void )
{
	int ww_enabled = WRAP_DISABLED;

	if (BufQSysFlags() & BUFFER_WORDWRAP)
	{
		ww_enabled = WRAP_ENABLED;
		if (BufQSysFlags() & BUFFER_WRAPCONTINUOUS)
			ww_enabled = WRAP_CONTINUOUS;
	}
	return ww_enabled;
}

PRIVATE int XNEAR
getTexWWMode( void )
{
	int ww_enabled = WRAP_DISABLED;

	if (TexQFlags() & FLAG_TEX_WRAP)
	{
		ww_enabled = WRAP_CONTINUOUS;
	}
	return ww_enabled;
}

/*
 ** TexWrapAll()
 *
 *  FILENAME: D:\CW32\TEX\TEX.C
 *
 *  PARAMETERS:
 *
 *  DESCRIPTION: TexWrappes all paragraphs into the buffer or into selection
 *
 *  RETURNS:
 *
 */

PUBLIC void DLL 
TexWrapAll( void )
{
	MarkID cmark, selmark, endmark;
	long endline;

	MsgMessage("TeX wrapping all, wait ... ");
	cmark = MarkAllocLocal();
	MarkSetColumnAdjust( cmark, FALSE, TRUE );
	//MarkSetColumnAdjust(MARK_SELECTION, FALSE, TRUE);
	selmark = MarkQSelBeginID();
	endmark = MarkQSelEndID();
	if ( selmark == MARK_INVALID )
	{
		MovTopBuf();
	}
	else
	{
		MarkGoto( endmark );
		MarkFree( endmark );
		endmark = MarkAllocLocal();
		MarkSetColumnAdjust( endmark, FALSE, TRUE );
		MarkGoto( selmark );
	}

	do
	{
		if ( endmark != MARK_INVALID )
		{
			selmark = MarkAllocLocal() ;
			MarkGoto( endmark );
			endline = BufQCurrentLine();
			//MsgNotify( StrItoA(endline,10) );
			MarkGoto( selmark );
			MarkFree( selmark );
			if ( endline < BufQCurrentLine() )
				break;
		}
		TexWrapParagraph();
 //		if( !MovNextChar(1L) )
 //		{
 //				break;
 //		}

		//MsgNotify("Paragraph wrapped");
	} while	( SrchFind("^[ \\t]\\c|\\n[ \\t]*\\n\\c", SEARCH_FORWARD | SEARCH_REGEX , NIL) );

	MarkGoto( cmark );
	MarkFree( cmark );
	MarkFree( endmark );
	MsgMessage("TeX Wrap all completed");
}

